@@ -23,18 +23,19 @@
 #include "libdm/misc/dm-logging.h"
 #include "base/memory/zalloc.h"
 
+#include "libdaemon/server/daemon-stray.h"
+
 #include <dlfcn.h>
 #include <pthread.h>
-#include <sys/file.h>
 #include <sys/stat.h>
 #include <sys/wait.h>
-#include <sys/time.h>
 #include <sys/resource.h>
 #include <signal.h>
 #include <arpa/inet.h>		/* for htonl, ntohl */
 #include <fcntl.h>		/* for musl libc */
 #include <unistd.h>
 #include <syslog.h>
+#include <sys/utsname.h>
 
 #ifdef __linux__
 /*
@@ -92,12 +93,14 @@ static const size_t THREAD_STACK_SIZE = 300 * 1024;
 /* Default idle exit timeout 1 hour (in seconds) */
 static const time_t DMEVENTD_IDLE_EXIT_TIMEOUT = 60 * 60;
 
-static int _debug_level = 0;
-static int _use_syslog = 1;
+/* Default grace period for thread cleanup 10 seconds */
+#define DMEVENTD_DEFAULT_GRACE_PERIOD 10
+static int _grace_period = DMEVENTD_DEFAULT_GRACE_PERIOD;
+
 static int _systemd_activation = 0;
 static int _foreground = 0;
-static int _restart = 0;
 static time_t _idle_since = 0;
+static const char *_exit_on = DEFAULT_DMEVENTD_EXIT_ON_PATH;
 static char **_initial_registrations = 0;
 
 /* FIXME Make configurable at runtime */
@@ -202,16 +205,17 @@ struct message_data {
 	char *dso_name;		/* Name of DSO. */
 	char *device_uuid;	/* Mapped device path. */
 	char *events_str;	/* Events string as fetched from message. */
-	enum dm_event_mask events_field;	/* Events bitfield. */
-	char *timeout_str;
+	unsigned events_field;	/* Events bitfield. */
 	uint32_t timeout_secs;
+	char *timeout_str;
 	struct dm_event_daemon_message *msg;	/* Pointer to message buffer. */
 };
 
 /* There are three states a thread can attain. */
 enum {
 	DM_THREAD_REGISTERING,	/* Registering, transitions to RUNNING */
-	DM_THREAD_RUNNING,	/* Working on events, transitions to DONE */
+	DM_THREAD_RUNNING,	/* Working on events, transitions to GRACE or DONE */
+	DM_THREAD_GRACE_PERIOD,	/* Thread awaits reuse for a grace period */
 	DM_THREAD_DONE		/* Terminated and cleanup is pending */
 };
 
@@ -235,12 +239,15 @@ struct thread_status {
 	} device;
 	int processing;		/* Set when event is being processed */
 
-	int status;		/* See DM_THREAD_{REGISTERING,RUNNING,DONE} */
+	int status;		/* See DM_THREAD_{REGISTERING,RUNNING,GRACE_PERIOD,DONE} */
 
 	int events;		/* bitfield for event filter. */
-	int current_events;	/* bitfield for occured events. */
+	int current_events;	/* bitfield for occurred events. */
 	struct dm_task *wait_task;
 	int pending;		/* Set when event filter change is pending */
+	int used;		/* Count thread reusage (for debugging) */
+	pthread_cond_t grace_cond;   /* Condition variable for grace period wait */
+	uint64_t inode;         /* Device path inode of monitored volume */
 	time_t next_time;
 	uint32_t timeout;
 	struct dm_list timeout_list;
@@ -270,7 +277,7 @@ static void _free_dso_data(struct dso_data *data)
 
 static struct dso_data *_alloc_dso_data(struct message_data *data)
 {
-	struct dso_data *ret = (typeof(ret)) zalloc(sizeof(*ret));
+	struct dso_data *ret = (__typeof__(ret)) zalloc(sizeof(*ret));
 
 	if (!ret)
 		return_NULL;
@@ -397,11 +404,17 @@ static void _free_thread_status(struct thread_status *thread)
 	_lib_put(thread->dso_data);
 	if (thread->wait_task)
 		dm_task_destroy(thread->wait_task);
+
+	/* Clean up grace period condition variable */
+	pthread_cond_destroy(&thread->grace_cond);
+
 	free(thread->device.uuid);
 	free(thread->device.name);
 	free(thread);
 }
 
+static int _lock_mutex(void);
+static int _unlock_mutex(void);
 /* Note: events_field must not be 0, ensured by caller */
 static struct thread_status *_alloc_thread_status(const struct message_data *data,
 						  struct dso_data *dso_data)
@@ -429,7 +442,13 @@ static struct thread_status *_alloc_thread_status(const struct message_data *dat
 	if (!(thread->device.name = strdup(data->device_uuid)))
 		goto_out;
 
-	/* runs ioctl and may register lvm2 pluging */
+	/* Initialize grace period condition variable */
+	if (pthread_cond_init(&thread->grace_cond, NULL)) {
+		log_error("Failed to initialize grace period condition variable.");
+		goto_out;
+	}
+
+	/* runs ioctl and may register lvm2 plugin */
 	thread->processing = 1;
 	thread->status = DM_THREAD_REGISTERING;
 
@@ -522,7 +541,7 @@ static int _fetch_string(char **ptr, char **src, const int delimiter)
 			*p = delimiter;
 			*src = p;
 		}
-		(*src)++; /* Skip delmiter, next field */
+		(*src)++; /* Skip delimiter, next field */
 	} else if ((len = strlen(*src))) {
 		/* No delimiter, item ends with '\0' */
 		if (!(*ptr = strdup(*src))) {
@@ -608,11 +627,14 @@ static int _fill_device_data(struct thread_status *ts)
 	if (!dm_task_run(dmt))
 		goto fail;
 
-	free(ts->device.name);
-	if (!(ts->device.name = strdup(dm_task_get_name(dmt))))
+	if (!dm_task_get_info(dmt, &dmi))
 		goto fail;
 
-	if (!dm_task_get_info(dmt, &dmi))
+	if (!dmi.exists)
+		goto fail;
+
+	free(ts->device.name);
+	if (!(ts->device.name = strdup(dm_task_get_name(dmt))))
 		goto fail;
 
 	ts->device.major = dmi.major;
@@ -628,9 +650,9 @@ fail:
 
 static struct dm_task *_get_device_status(struct thread_status *ts)
 {
-	struct dm_task *dmt = dm_task_create(DM_DEVICE_STATUS);
+	struct dm_task *dmt;
 
-	if (!dmt)
+	if (!(dmt = dm_task_create(DM_DEVICE_STATUS)))
 		return_NULL;
 
 	if (!dm_task_set_uuid(dmt, ts->device.uuid)) {
@@ -650,6 +672,60 @@ static struct dm_task *_get_device_status(struct thread_status *ts)
 	return dmt;
 }
 
+static uint64_t _get_device_inode(struct thread_status *ts)
+{
+	static int _kernel_major = -1;
+	struct utsname uts;
+	struct stat buf;
+	char path[PATH_MAX];
+
+	/* Get kernel version to determine path format */
+	if (_kernel_major < 0) {
+		_kernel_major = 0;
+		if (uname(&uts))
+			log_sys_debug("uname", "");
+		else if (sscanf(uts.release, "%d", &_kernel_major) != 1)
+			log_debug("Cannot parse kernel version from %s.", uts.release);
+	}
+
+	if (_kernel_major >= 3) {
+		/* Use sysfs path with major:minor format for modern kernels */
+		if (dm_snprintf(path, sizeof(path), "%sdev/block/%d:%d",
+				dm_sysfs_dir(), ts->device.major, ts->device.minor) < 0)
+			return_0;
+	} else {
+		/* Use /dev/mapper/name device path for kernel version <3.
+		 * Older kernels do not change inode numbers for devices!
+		 * Relies on correct files in /dev/mapper directory.
+		 */
+		if (dm_snprintf(path, sizeof(path), "%s/%s",
+				dm_dir(), ts->device.name) < 0)
+			return_0;
+	}
+
+	if (stat(path, &buf) < 0) {
+		log_sys_debug("stat", path);
+		if (_kernel_major >= 3)
+			return 0;
+
+		/* Since monitoring is not synchronized with udev
+		 * symlink may not exists, so also try /dev/dm-X */
+		if (dm_snprintf(path, sizeof(path), "%s/../dm-%d",
+				dm_dir(), ts->device.minor) < 0)
+			return_0;
+
+		if (stat(path, &buf) < 0) {
+			log_sys_debug("stat", path);
+			return 0;
+		}
+	}
+
+	log_debug("Device %s with inode %" PRIu64 " (kernel %d).",
+		  path, (uint64_t) buf.st_ino, _kernel_major);
+
+	return (uint64_t) buf.st_ino;
+}
+
 /*
  * Find an existing thread for a device.
  *
@@ -666,6 +742,22 @@ static struct thread_status *_lookup_thread_status(struct message_data *data)
 	return NULL;
 }
 
+static struct thread_status *_lookup_grace_thread_status(struct message_data *data)
+{
+	struct thread_status *thread;
+
+	dm_list_iterate_items(thread, &_thread_registry_unused)
+		if ((thread->status == DM_THREAD_GRACE_PERIOD) &&
+		    !strcmp(data->device_uuid, thread->device.uuid) &&
+		    !strcmp(data->dso_name, thread->dso_data->dso_name) &&
+		    (thread->inode == _get_device_inode(thread))) {
+			DEBUGLOG("Found reusable thread %x in grace period.",(int)thread->thread);
+			return thread;
+		}
+
+	return NULL;
+}
+
 static int _get_status(struct message_data *message_data)
 {
 	struct dm_event_daemon_message *msg = message_data->msg;
@@ -678,10 +770,19 @@ static int _get_status(struct message_data *message_data)
 	char **buffers;
 	char *message;
 
+	if (!message_data->id)
+		return -EINVAL;
+
 	_lock_mutex();
-	count = dm_list_size(&_thread_registry);
+	if (!(count = dm_list_size(&_thread_registry))) {
+		_unlock_mutex();
+		ret = 0;        /* no monitored devices */
+		goto out;
+	}
+
 	buffers = alloca(sizeof(char*) * count);
 	dm_list_iterate_items(thread, &_thread_registry) {
+		/* coverity[overflow_sink] - only positive 'current' is used */
 		if ((current = dm_asprintf(buffers + i, "0:%d %s %s %u %" PRIu32 ";",
 					   i, thread->dso_data->dso_name,
 					   thread->device.uuid, thread->events,
@@ -690,6 +791,7 @@ static int _get_status(struct message_data *message_data)
 			goto out;
 		}
 		++i;
+		/* coverity[overflow] - only positive 'current' is used */
 		size += current; /* count with trailing '\0' */
 	}
 	_unlock_mutex();
@@ -720,12 +822,18 @@ static int _get_status(struct message_data *message_data)
 static int _get_parameters(struct message_data *message_data) {
 	struct dm_event_daemon_message *msg = message_data->msg;
 	int size;
+	char idle_buf[32] = "";
+
+	if (_idle_since)
+		(void)dm_snprintf(idle_buf, sizeof(idle_buf), " idle=%lu", (long unsigned) (time(NULL) - _idle_since));
 
 	free(msg->data);
-	if ((size = dm_asprintf(&msg->data, "%s pid=%d daemon=%s exec_method=%s",
+	if ((size = dm_asprintf(&msg->data, "%s pid=%d daemon=%s exec_method=%s exit_on=\"%s\"%s",
 				message_data->id, getpid(),
 				_foreground ? "no" : "yes",
-				_systemd_activation ? "systemd" : "direct")) < 0) {
+				_systemd_activation ? "systemd" : "direct",
+				_exit_on,
+				idle_buf)) < 0) {
 		stack;
 		return -ENOMEM;
 	}
@@ -778,7 +886,11 @@ static void *_timeout_thread(void *unused __attribute__((unused)))
 			if (thread->next_time <= curr_time) {
 				thread->next_time = curr_time + thread->timeout;
 				_lock_mutex();
-				if (thread->processing) {
+				if (thread->status != DM_THREAD_RUNNING) {
+					/* Skip wake up of non running thread (i.e. in grace period) */
+					log_debug("Skipping SIGALRM to non running Thr %x for timeout.",
+						  (int) thread->thread);
+				} else if (thread->processing) {
 					/* Cannot signal processing monitoring thread */
 					log_debug("Skipping SIGALRM to processing Thr %x for timeout.",
 						  (int) thread->thread);
@@ -797,8 +909,8 @@ static void *_timeout_thread(void *unused __attribute__((unused)))
 				timeout.tv_sec = thread->next_time;
 		}
 
-		pthread_cond_timedwait(&_timeout_cond, &_timeout_mutex,
-				       &timeout);
+		(void) pthread_cond_timedwait(&_timeout_cond, &_timeout_mutex,
+					      &timeout);
 	}
 
 	DEBUGLOG("Timeout thread finished.");
@@ -865,6 +977,43 @@ enum {
 	DM_WAIT_FATAL
 };
 
+/* Reset pending signal for a task/thread */
+static int _reset_pending_signal(int signal)
+{
+	sigset_t prev_mask, mask;
+	struct sigaction prev_act, act = { .sa_handler = SIG_IGN };
+
+	sigemptyset(&act.sa_mask);
+
+	sigemptyset(&prev_mask);
+
+	sigemptyset(&mask);
+	sigaddset(&mask, signal);
+
+	if (pthread_sigmask(SIG_SETMASK, &mask, &prev_mask) != 0) {
+		log_sys_error("pthread_sigmask", "ignore signal");
+		return 0; /* What better */
+	}
+
+	if (sigaction(signal, &act, &prev_act) < 0) {
+		log_sys_error("sigaction", "ignore signal");
+		return 0;
+	}
+
+	if (sigaction(signal, &prev_act, NULL) < 0) {
+		log_sys_error("sigaction", "restore signal");
+		return 0;
+	}
+
+	/* And also restore the process's original sigmask */
+	if (pthread_sigmask(SIG_SETMASK, &prev_mask, NULL) < 0) {
+		log_sys_error("pthread_sigmask", "restore signal");
+		return 0;
+	}
+
+	return 1;
+}
+
 /* Wait on a device until an event occurs. */
 static int _event_wait(struct thread_status *thread)
 {
@@ -887,6 +1036,9 @@ static int _event_wait(struct thread_status *thread)
 	}
 
 	if (dm_task_run(thread->wait_task)) {
+		/* Recheck device info whether is still exists */
+		if (!_fill_device_data(thread))
+			goto disappeared; /* device is gone... */
 		thread->current_events |= DM_EVENT_DEVICE_ERROR;
 		ret = DM_WAIT_INTR;
 		/* Update event_nr */
@@ -895,6 +1047,7 @@ static int _event_wait(struct thread_status *thread)
 	} else {
 		switch (dm_task_get_errno(thread->wait_task)) {
 		case ENXIO:
+disappeared:
 			log_error("%s disappeared, detaching.",
 				  thread->device.name);
 			ret = DM_WAIT_FATAL;
@@ -951,7 +1104,8 @@ static void _do_process_event(struct thread_status *thread)
 	if (!task)
 		log_error("Lost event in Thr %x.", (int)thread->thread);
 	else {
-		thread->dso_data->process_event(task, thread->current_events, &(thread->dso_private));
+		thread->dso_data->process_event(task, (enum dm_event_mask) thread->current_events,
+						&(thread->dso_private));
 		if (task != thread->wait_task)
 			dm_task_destroy(task);
 	}
@@ -963,6 +1117,12 @@ static void _thread_unused(struct thread_status *thread)
 	LINK(thread, &_thread_registry_unused);
 }
 
+static void _thread_used(struct thread_status *thread)
+{
+	UNLINK_THREAD(thread);
+	LINK_THREAD(thread);
+}
+
 /* Thread cleanup handler to unregister device. */
 static void _monitor_unregister(void *arg)
 {
@@ -984,6 +1144,7 @@ static void _monitor_unregister(void *arg)
 	DEBUGLOG("Unregistering monitor for %s.", thread->device.name);
 	_unregister_for_timeout(thread);
 
+	/* coverity[missing_lock] no missing lock here */
 	if ((thread->status != DM_THREAD_REGISTERING) &&
 	    !_do_unregister_device(thread))
 		log_error("%s: %s unregister failed.", __func__,
@@ -994,34 +1155,15 @@ static void _monitor_unregister(void *arg)
 	_lock_mutex();
 	thread->status = DM_THREAD_DONE; /* Last access to thread memory! */
 	_unlock_mutex();
+	if (_exit_now)  /* Exit is already in-progress, wake-up sleeping select() */
+		kill(getpid(), SIGINT);
 }
 
-/* Device monitoring thread. */
-static void *_monitor_thread(void *arg)
+static int _monitor_events(struct thread_status *thread)
 {
-	struct thread_status *thread = arg;
-	int ret;
+	int ret = 0;
 	sigset_t pendmask;
 
-	pthread_setcanceltype(PTHREAD_CANCEL_DEFERRED, NULL);
-	pthread_cleanup_push(_monitor_unregister, thread);
-
-	if (!_fill_device_data(thread)) {
-		log_error("Failed to fill device data for %s.", thread->device.uuid);
-		_lock_mutex();
-		goto out;
-	}
-
-	if (!_do_register_device(thread)) {
-		log_error("Failed to register device %s.", thread->device.name);
-		_lock_mutex();
-		goto out;
-	}
-
-	_lock_mutex();
-	thread->status = DM_THREAD_RUNNING;
-	thread->processing = 0;
-
 	/* Loop awaiting/analyzing device events. */
 	while (thread->events) {
 
@@ -1040,9 +1182,9 @@ static void *_monitor_thread(void *arg)
 			_unlock_mutex();
 
 			_do_process_event(thread);
-			thread->current_events = 0; /* Current events processed */
 
 			_lock_mutex();
+			thread->current_events = 0; /* Current events processed */
 			thread->processing = 0;
 
 			/*
@@ -1067,11 +1209,102 @@ static void *_monitor_thread(void *arg)
 				break;
 		}
 	}
+
+	return ret;
+}
+
+/* Thread awaits condition wake up for a grace period */
+static void _monitor_grace_period_wait(struct thread_status *thread)
+{
+	struct timespec grace_timeout = { .tv_sec = time(NULL) + _grace_period };
+
+	DEBUGLOG("Thread %x entering grace period for %d seconds.",
+		 (int)thread->thread, _grace_period);
+
+	/* Wait on per-thread condition variable with global mutex */
+	while (!_exit_now && !thread->events &&
+	       (ETIMEDOUT != pthread_cond_timedwait(&thread->grace_cond,
+						    &_global_mutex, &grace_timeout)))
+		/* Waiting */;
+
+	DEBUGLOG("Thread %x wakeup grace period.", (int)thread->thread);
+}
+
+/* Device monitoring thread. */
+static void *_monitor_thread(void *arg)
+{
+	struct thread_status *thread = arg;
+	int ret;
+
+	pthread_setcanceltype(PTHREAD_CANCEL_DEFERRED, NULL);
+	pthread_cleanup_push(_monitor_unregister, thread);
+
+	if (!_fill_device_data(thread)) {
+		log_error("Failed to fill device data for %s.", thread->device.uuid);
+		_lock_mutex();
+		goto out;
+	}
+
+	/* Now with resolved major:minor store also device inode */
+	thread->inode = _get_device_inode(thread);
+
+	if (!_do_register_device(thread)) {
+		log_error("Failed to register device %s.", thread->device.name);
+		_lock_mutex();
+		goto out;
+	}
+
+	_lock_mutex();
+
+	/* Main monitoring loop with grace period support */
+	while (thread->events) {
+		DEBUGLOG("Monitoring %s with Thr %x  (events: %x, used: %d).",
+			 thread->device.name, (int)thread->thread,
+			 thread->events, thread->used);
+
+		thread->status = DM_THREAD_RUNNING;
+		thread->processing = 0;
+		thread->used++;
+
+		ret = _monitor_events(thread);
+
+		/* No grace period when set to 0
+		 * or there were left some processing events which is an error state
+		 * or there is on going exit
+		 * or there was fatal error while waiting for some event */
+		if (!_grace_period || thread->events || _exit_now || (ret == DM_WAIT_FATAL))
+			break;
+
+		/* Before restarting event loop reset any pending SIGALRM signal */
+		if (!_reset_pending_signal(SIGALRM)) {
+			stack;
+			break; /* Something is wrong... */
+		}
+
+		thread->current_events = 0;
+		thread->status = DM_THREAD_GRACE_PERIOD;	/* No events - enter grace period */
+		_thread_unused(thread);
+
+		DEBUGLOG("Gracing %s with Thr %x  (events: %x, used: %d).",
+			 thread->device.name, (int)thread->thread,
+			 thread->events, thread->used);
+
+		_monitor_grace_period_wait(thread);
+		_unlock_mutex();
+
+		pthread_mutex_lock(&_timeout_mutex);
+		thread->next_time = time(NULL) + thread->timeout;
+		pthread_mutex_unlock(&_timeout_mutex);
+
+		_lock_mutex();
+		_thread_used(thread);
+	}
 out:
 	/* ';' fixes gcc compilation problem with older pthread macros
 	 * "label at end of compound statement" */
 	;
 
+	/* coverity[lock_order] _global_mutex is kept locked */
 	pthread_cleanup_pop(1);
 
 	return NULL;
@@ -1094,6 +1327,15 @@ static int _update_events(struct thread_status *thread, int events)
 	thread->events = events;
 	thread->pending = DM_EVENT_REGISTRATION_PENDING;
 
+	/* If needed, wake up thread waiting in grace period */
+	if ((events || _exit_now) && (thread->status == DM_THREAD_GRACE_PERIOD)) {
+		DEBUGLOG("Waking up thread %x waiting in grace period (events=%x).",
+			 (int)thread->thread, events);
+		/* Signal per-thread condition variable while holding global mutex */
+		pthread_cond_signal(&thread->grace_cond);
+		return 0;
+	}
+
 	/* Only non-processing threads can be notified */
 	if (!thread->processing) {
 		DEBUGLOG("Sending SIGALRM to wakeup Thr %x.", (int)thread->thread);
@@ -1108,9 +1350,7 @@ static int _update_events(struct thread_status *thread, int events)
 		}
 	}
 
-	/* Threads with no events has to be moved to unused */
-	if (!thread->events)
-		_thread_unused(thread);
+	/* Threads with no events will enter grace period in their main loop */
 
 	return -ret;
 }
@@ -1157,6 +1397,36 @@ static int _unregister_for_event(struct message_data *message_data)
 	return ret;
 }
 
+static void _unregister_all_threads(void)
+{
+	struct thread_status *thread, *tmp;
+
+	_lock_mutex();
+
+	dm_list_iterate_items_safe(thread, tmp, &_thread_registry)
+		_update_events(thread, 0);
+
+	_unlock_mutex();
+}
+
+static void _wait_for_new_pid(void)
+{
+	unsigned long st_ino = 0;
+	struct stat st;
+	int i;
+
+	for (i = 0; i < 400000; ++i) {
+		if (lstat(DMEVENTD_PIDFILE, &st) == 0) {
+			if (!st_ino)
+				st_ino = st.st_ino;
+			else if (st_ino != st.st_ino)
+				break; /* different pidfile */
+		} else if (errno == ENOENT)
+			break; /* pidfile is removed */
+		usleep(100);
+	}
+}
+
 /*
  * Register for an event.
  *
@@ -1182,7 +1452,8 @@ static int _register_for_event(struct message_data *message_data)
 
 	_lock_mutex();
 
-	if ((thread = _lookup_thread_status(message_data))) {
+	if ((thread = _lookup_thread_status(message_data)) ||
+	    (thread = _lookup_grace_thread_status(message_data))) {
 		/* OR event # into events bitfield. */
 		ret = _update_events(thread, (thread->events | message_data->events_field));
 	} else {
@@ -1296,11 +1567,13 @@ static int _get_registered_dev(struct message_data *message_data, int next)
 
 	/*
 	 * If we didn't get a match, try the threads waiting to be deleted.
+	 * Threads in grace period are skipped.
 	 * FIXME Do something similar if 'next' is set.
 	 */
 	if (!hit && !next)
 		dm_list_iterate_items(thread, &_thread_registry_unused)
-			if (_want_registered_device(message_data->dso_name,
+			if ((thread->status != DM_THREAD_GRACE_PERIOD) &&
+			    _want_registered_device(message_data->dso_name,
 						    message_data->device_uuid, thread)) {
 				hit = thread;
 				goto reg;
@@ -1385,8 +1658,8 @@ static int _open_fifo(const char *path)
 {
 	struct stat st;
 	int fd = -1;
- 
- 	/*
+
+	/*
 	 * FIXME Explicitly verify the code's requirement that path is secure:
 	 * - All parent directories owned by root without group/other write access unless sticky.
 	 */
@@ -1400,7 +1673,8 @@ static int _open_fifo(const char *path)
 	} else if (!S_ISFIFO(st.st_mode) || st.st_uid ||
 		   (st.st_mode & (S_IEXEC | S_IRWXG | S_IRWXO))) {
 		log_warn("WARNING: %s has wrong attributes: Replacing.", path);
-		if (unlink(path)) {
+		/* coverity[toctou]  don't care, path is going to be recreated */
+		if (unlink(path) && (errno != ENOENT)) {
 			log_sys_error("unlink", path);
 			return -1;
 		}
@@ -1408,6 +1682,7 @@ static int _open_fifo(const char *path)
 
 	/* Create fifo. */
 	(void) dm_prepare_selinux_context(path, S_IFIFO);
+	/* coverity[toctou]  revalidating things again */
 	if ((mkfifo(path, 0600) == -1) && errno != EEXIST) {
 		log_sys_error("mkfifo", path);
 		(void) dm_prepare_selinux_context(NULL, 0);
@@ -1636,7 +1911,7 @@ static int _do_process_request(struct dm_event_daemon_message *msg)
 	} else
 		ret = _handle_request(msg, &message_data);
 
-	msg->cmd = ret;
+	msg->cmd = (uint32_t)ret;
 	if (!msg->data)
 		msg->size = dm_asprintf(&(msg->data), "%s %s", message_data.id, strerror(-ret));
 
@@ -1673,9 +1948,9 @@ static void _process_request(struct dm_event_fifos *fifos)
 	free(msg.data);
 
 	if (cmd == DM_EVENT_CMD_DIE) {
-		if (unlink(DMEVENTD_PIDFILE))
-			log_sys_error("unlink", DMEVENTD_PIDFILE);
-		_exit(0);
+		_exit_now = DM_SCHEDULED_EXIT; /* No grace period */
+		_unregister_all_threads();
+		log_info("dmeventd exiting for restart.");
 	}
 }
 
@@ -1722,7 +1997,7 @@ static void _cleanup_unused_threads(void)
 		DEBUGLOG("Destroying Thr %x.", (int)thread->thread);
 
 		if (pthread_join(thread->thread, NULL))
-			log_sys_error("pthread_join", "");
+			log_sys_debug("pthread_join", "");
 
 		_free_thread_status(thread);
 		_lock_mutex();
@@ -1753,7 +2028,7 @@ static void _init_thread_signals(void)
 	sigdelset(&my_sigset, SIGQUIT);
 
 	if (pthread_sigmask(SIG_BLOCK, &my_sigset, NULL))
-		log_sys_error("pthread_sigmask", "SIG_BLOCK");
+		log_sys_debug("pthread_sigmask", "SIG_BLOCK");
 }
 
 /*
@@ -1765,7 +2040,8 @@ static void _init_thread_signals(void)
  */
 static void _exit_handler(int sig __attribute__((unused)))
 {
-	_exit_now = DM_SIGNALED_EXIT;
+	if (!_exit_now)
+		_exit_now = DM_SIGNALED_EXIT;
 }
 
 #ifdef __linux__
@@ -1781,7 +2057,7 @@ static int _set_oom_adj(const char *oom_adj_path, int val)
 	fprintf(fp, "%i", val);
 
 	if (dm_fclose(fp))
-		log_sys_error("fclose", oom_adj_path);
+		log_sys_debug("fclose", oom_adj_path);
 
 	return 1;
 }
@@ -1795,11 +2071,11 @@ static int _protect_against_oom_killer(void)
 
 	if (stat(OOM_ADJ_FILE, &st) == -1) {
 		if (errno != ENOENT)
-			log_sys_error("stat", OOM_ADJ_FILE);
+			log_sys_debug("stat", OOM_ADJ_FILE);
 
 		/* Try old oom_adj interface as a fallback */
 		if (stat(OOM_ADJ_FILE_OLD, &st) == -1) {
-			log_sys_error("stat", OOM_ADJ_FILE_OLD);
+			log_sys_debug("stat", OOM_ADJ_FILE_OLD);
 			return 1;
 		}
 
@@ -1887,26 +2163,30 @@ out:
 
 static void _remove_files_on_exit(void)
 {
-	if (unlink(DMEVENTD_PIDFILE))
-		log_sys_error("unlink", DMEVENTD_PIDFILE);
+	if (unlink(DMEVENTD_PIDFILE) && (errno != ENOENT))
+		log_sys_debug("unlink", DMEVENTD_PIDFILE);
 
 	if (!_systemd_activation) {
-		if (unlink(DM_EVENT_FIFO_CLIENT))
-			log_sys_error("unlink", DM_EVENT_FIFO_CLIENT);
+		if (unlink(DM_EVENT_FIFO_CLIENT) && (errno != ENOENT))
+			log_sys_debug("unlink", DM_EVENT_FIFO_CLIENT);
 
-		if (unlink(DM_EVENT_FIFO_SERVER))
-			log_sys_error("unlink", DM_EVENT_FIFO_SERVER);
+		if (unlink(DM_EVENT_FIFO_SERVER) && (errno != ENOENT))
+			log_sys_debug("unlink", DM_EVENT_FIFO_SERVER);
 	}
 }
 
 static void _daemonize(void)
 {
-	int child_status;
-	int fd;
+	int child_status, null_fd;
 	pid_t pid;
-	struct rlimit rlim;
 	struct timeval tval;
 	sigset_t my_sigset;
+	struct custom_fds custom_fds = {
+		/* Do not close fds preloaded by systemd! */
+		.out = (_systemd_activation) ? SD_FD_FIFO_SERVER : -1,
+		.err = -1,
+		.report = (_systemd_activation) ? SD_FD_FIFO_CLIENT : -1,
+	};
 
 	sigemptyset(&my_sigset);
 	if (sigprocmask(SIG_SETMASK, &my_sigset, NULL) < 0) {
@@ -1950,33 +2230,28 @@ static void _daemonize(void)
 	if (chdir("/"))
 		exit(EXIT_CHDIR_FAILURE);
 
-	if (getrlimit(RLIMIT_NOFILE, &rlim) < 0)
-		fd = 256;	/* just have to guess */
-	else
-		fd = rlim.rlim_cur;
+	daemon_close_stray_fds("dmeventd", 0, -1, &custom_fds);
 
-	for (--fd; fd >= 0; fd--) {
-#ifdef __linux__
-		/* Do not close fds preloaded by systemd! */
-		if (_systemd_activation &&
-		    (fd == SD_FD_FIFO_SERVER || fd == SD_FD_FIFO_CLIENT))
-			continue;
-#endif
-		(void) close(fd);
-	}
-
-	if ((open("/dev/null", O_RDONLY) < 0) ||
-	    (open("/dev/null", O_WRONLY) < 0) ||
-	    (open("/dev/null", O_WRONLY) < 0))
+	if ((null_fd = open("/dev/null", O_RDWR)) < 0)
 		exit(EXIT_DESC_OPEN_FAILURE);
 
+	if ((dup2(null_fd, STDIN_FILENO) == -1) ||
+	    (dup2(null_fd, STDOUT_FILENO) == -1) ||
+	    (dup2(null_fd, STDERR_FILENO) == -1))
+		exit(EXIT_DESC_OPEN_FAILURE);
+
+	if ((null_fd > STDERR_FILENO) && close(null_fd))
+		exit(EXIT_DESC_CLOSE_FAILURE);
+
 	setsid();
+
+	/* coverity[leaked_handle] 'null_fd' handle is not leaking */
 }
 
 static int _reinstate_registrations(struct dm_event_fifos *fifos)
 {
 	static const char _failed_parsing_msg[] = "Failed to parse existing event registration.\n";
-	static const char *_delim = " ";
+	static const char _delim[] = " ";
 	struct dm_event_daemon_message msg = { 0 };
 	char *endp, *dso_name, *dev_name, *mask, *timeout;
 	unsigned long mask_value, timeout_value;
@@ -2026,28 +2301,94 @@ static int _reinstate_registrations(struct dm_event_fifos *fifos)
 	return 1;
 }
 
-static void _restart_dmeventd(void)
+static int _info_dmeventd(const char *name, struct dm_event_fifos *fifos)
+{
+	struct dm_event_daemon_message msg = { 0 };
+	int i, count = 0;
+	char *line;
+	int version;
+	int ret = 0;
+
+	if (!dm_daemon_is_running(DMEVENTD_PIDFILE)) {
+		fprintf(stderr, "No running dmeventd instance for status query.\n");
+		return 0;
+	}
+
+	/* Get the list of registrations from the running daemon. */
+	if (!init_fifos(fifos)) {
+		fprintf(stderr, "Could not initiate communication with existing dmeventd.\n");
+		return 0;
+	}
+
+	if (!dm_event_get_version(fifos, &version)) {
+		fprintf(stderr, "Could not communicate with existing dmeventd.\n");
+		goto out;
+	}
+
+	if (version < 1) {
+		fprintf(stderr, "The running dmeventd instance is too old.\n"
+			"Protocol version %d (required: 1). Action cancelled.\n", version);
+		goto out;
+	}
+
+	if (daemon_talk(fifos, &msg, DM_EVENT_CMD_GET_STATUS, "-", "-", 0, 0)) {
+		fprintf(stderr, "Failed to acquire status from existing dmeventd.\n");
+		goto out;
+	}
+
+	line = strchr(msg.data, ' ') + 1;
+	for (i = 0; msg.data[i]; ++i)
+		if (msg.data[i] == ';') {
+			msg.data[i] = 0;
+			if (!count)
+				printf("%s is monitoring:\n", name);
+			printf("%s\n", line);
+			line = msg.data + i + 1;
+			++count;
+		}
+
+	free(msg.data);
+
+	if (!count)
+		printf("%s does not monitor any device.\n", name);
+
+	if (version >= 2) {
+		if (daemon_talk(fifos, &msg, DM_EVENT_CMD_GET_PARAMETERS, "-", "-", 0, 0)) {
+			fprintf(stderr, "Failed to acquire parameters from existing dmeventd.\n");
+			goto out;
+		}
+		printf("%s internal status: %s\n", name, msg.data);
+		free(msg.data);
+	}
+
+	ret = 1;
+out:
+	fini_fifos(fifos);
+
+	return ret;
+}
+
+/* Return   0 - fail, 1 - success, 2 - continue */
+static int _restart_dmeventd(struct dm_event_fifos *fifos)
 {
-	struct dm_event_fifos fifos = {
-		.client = -1,
-		.server = -1,
-		/* FIXME Make these either configurable or depend directly on dmeventd_path */
-		.client_path = DM_EVENT_FIFO_CLIENT,
-		.server_path = DM_EVENT_FIFO_SERVER
-	};
 	struct dm_event_daemon_message msg = { 0 };
 	int i, count = 0;
 	char *message;
 	int version;
 	const char *e;
 
-	/* Get the list of registrations from the running daemon. */
-	if (!init_fifos(&fifos)) {
-		fprintf(stderr, "WARNING: Could not initiate communication with existing dmeventd.\n");
-		exit(EXIT_FAILURE);
+	if (!dm_daemon_is_running(DMEVENTD_PIDFILE)) {
+		fprintf(stderr, "WARNING: Could not find running dmeventd associated with pid file %s.\n", DMEVENTD_PIDFILE);
+		return 0;
 	}
 
-	if (!dm_event_get_version(&fifos, &version)) {
+	/* Get the list of registrations from the running daemon. */
+	if (!init_fifos(fifos)) {
+		fprintf(stderr, "WARNING: Could not initiate communication with existing dmeventd.\n");
+		return 0;
+	}
+
+	if (!dm_event_get_version(fifos, &version)) {
 		fprintf(stderr, "WARNING: Could not communicate with existing dmeventd.\n");
 		goto bad;
 	}
@@ -2059,7 +2400,7 @@ static void _restart_dmeventd(void)
 		goto bad;
 	}
 
-	if (daemon_talk(&fifos, &msg, DM_EVENT_CMD_GET_STATUS, "-", "-", 0, 0))
+	if (daemon_talk(fifos, &msg, DM_EVENT_CMD_GET_STATUS, "-", "-", 0, 0))
 		goto bad;
 
 	message = strchr(msg.data, ' ') + 1;
@@ -2069,7 +2410,7 @@ static void _restart_dmeventd(void)
 			++count;
 		}
 
-	if (!(_initial_registrations = malloc(sizeof(char*) * (count + 1)))) {
+	if (!(_initial_registrations = zalloc(sizeof(char*) * (count + 1)))) {
 		fprintf(stderr, "Memory allocation registration failed.\n");
 		goto bad;
 	}
@@ -2081,10 +2422,9 @@ static void _restart_dmeventd(void)
 		}
 		message += strlen(message) + 1;
 	}
-	_initial_registrations[count] = NULL;
 
 	if (version >= 2) {
-		if (daemon_talk(&fifos, &msg, DM_EVENT_CMD_GET_PARAMETERS, "-", "-", 0, 0)) {
+		if (daemon_talk(fifos, &msg, DM_EVENT_CMD_GET_PARAMETERS, "-", "-", 0, 0)) {
 			fprintf(stderr, "Failed to acquire parameters from old dmeventd.\n");
 			goto bad;
 		}
@@ -2104,7 +2444,7 @@ static void _restart_dmeventd(void)
 	}
 #endif
 
-	if (daemon_talk(&fifos, &msg, DM_EVENT_CMD_DIE, "-", "-", 0, 0)) {
+	if (daemon_talk(fifos, &msg, DM_EVENT_CMD_DIE, "-", "-", 0, 0)) {
 		fprintf(stderr, "Old dmeventd refused to die.\n");
 		goto bad;
 	}
@@ -2113,52 +2453,56 @@ static void _restart_dmeventd(void)
 	    ((e = getenv(SD_ACTIVATION_ENV_VAR_NAME)) && strcmp(e, "1")))
 		_systemd_activation = 1;
 
-	for (i = 0; i < 10; ++i) {
-		if ((access(DMEVENTD_PIDFILE, F_OK) == -1) && (errno == ENOENT))
-			break;
-		usleep(10);
-	}
+	fini_fifos(fifos);
 
-	if (!_systemd_activation) {
-		fini_fifos(&fifos);
-		return;
-	}
+	/* Give a few seconds dmeventd to finish */
+	_wait_for_new_pid();
+
+	if (!_systemd_activation)
+		return 2; // continue with dmeventd start up
 
 	/* Reopen fifos. */
-	fini_fifos(&fifos);
-	if (!init_fifos(&fifos)) {
+	if (!init_fifos(fifos)) {
 		fprintf(stderr, "Could not initiate communication with new instance of dmeventd.\n");
-		exit(EXIT_FAILURE);
+		return 0;
 	}
 
-	if (!_reinstate_registrations(&fifos)) {
+	if (!_reinstate_registrations(fifos)) {
 		fprintf(stderr, "Failed to reinstate monitoring with new instance of dmeventd.\n");
 		goto bad;
 	}
 
-	fini_fifos(&fifos);
-	exit(EXIT_SUCCESS);
+	fini_fifos(fifos);
+	return 1;
 bad:
-	fini_fifos(&fifos);
-	exit(EXIT_FAILURE);
+	fini_fifos(fifos);
+	return 0;
 }
 
 static void _usage(char *prog, FILE *file)
 {
 	fprintf(file, "Usage:\n"
-		"%s [-d [-d [-d]]] [-f] [-h] [-l] [-R] [-V] [-?]\n\n"
+		"%s [-d [-d [-d]]] [-e path] [-g seconds] [-f] [-h] [i] [-l] [-R] [-V] [-?]\n\n"
 		"   -d       Log debug messages to syslog (-d, -dd, -ddd)\n"
+		"   -e       Select a file path checked on exit\n"
+		"   -g       Grace period for thread cleanup (0-300 seconds, default: %d)\n"
 		"   -f       Don't fork, run in the foreground\n"
 		"   -h       Show this help information\n"
+		"   -i       Query running instance of dmeventd for info\n"
 		"   -l       Log to stdout,stderr instead of syslog\n"
 		"   -?       Show this help information on stderr\n"
 		"   -R       Restart dmeventd\n"
-		"   -V       Show version of dmeventd\n\n", prog);
+		"   -V       Show version of dmeventd\n\n", prog,
+		_grace_period);
 }
 
 int main(int argc, char *argv[])
 {
 	signed char opt;
+	int debug_level = 0;
+	int info = 0;
+	int restart = 0;
+	int use_syslog = 1;
 	struct dm_event_fifos fifos = {
 		.client = -1,
 		.server = -1,
@@ -2166,39 +2510,65 @@ int main(int argc, char *argv[])
 		.server_path = DM_EVENT_FIFO_SERVER
 	};
 	time_t now, idle_exit_timeout = DMEVENTD_IDLE_EXIT_TIMEOUT;
-	opterr = 0;
-	optind = 0;
 
-	while ((opt = getopt(argc, argv, "?fhVdlR")) != EOF) {
+	optopt = optind = opterr = 0;
+	optarg = (char*) "";
+	while ((opt = getopt(argc, argv, ":?e:g:fhiVdlR")) != EOF) {
 		switch (opt) {
 		case 'h':
 			_usage(argv[0], stdout);
-			exit(EXIT_SUCCESS);
+			return EXIT_SUCCESS;
 		case '?':
 			_usage(argv[0], stderr);
-			exit(EXIT_SUCCESS);
+			return EXIT_SUCCESS;
+		case 'i':
+			info++;
+			break;
 		case 'R':
-			_restart++;
+			restart++;
+			break;
+		case 'e':
+			if (strchr(optarg, '"')) {
+				fprintf(stderr, "dmeventd: option -e does not accept path \"%s\" with '\"' character.\n", optarg);
+				return EXIT_FAILURE;
+			}
+			_exit_on=optarg;
+			break;
+		case 'g':
+			_grace_period = atoi(optarg);
+			if (_grace_period < 0 || _grace_period > 300) {
+				fprintf(stderr, "dmeventd: grace period must be between 0 and 300 seconds.\n");
+				return EXIT_FAILURE;
+			}
 			break;
 		case 'f':
 			_foreground++;
 			break;
 		case 'd':
-			_debug_level++;
+			debug_level++;
 			break;
 		case 'l':
-			_use_syslog = 0;
+			use_syslog = 0;
 			break;
 		case 'V':
 			printf("dmeventd version: %s\n", DM_LIB_VERSION);
-			exit(EXIT_SUCCESS);
+			return EXIT_SUCCESS;
+		case ':':
+			fprintf(stderr, "dmeventd: option -%c requires an argument.\n", optopt);
+			return EXIT_FAILURE;
 		}
 	}
 
-	if (!_foreground && !_use_syslog) {
-		printf("WARNING: Ignoring logging to stdout, needs options -f\n");
-		_use_syslog = 1;
+	if (info) {
+		_foreground = 1;
+		use_syslog = 0;
 	}
+
+	if (!_foreground && !use_syslog) {
+		printf("WARNING: Ignoring logging to stdout, needs options -f\n");
+		use_syslog = 1;
+	}
+
 	/*
 	 * Switch to C locale to avoid reading large locale-archive file
 	 * used by some glibc (on some distributions it takes over 100MB).
@@ -2207,21 +2577,29 @@ int main(int argc, char *argv[])
 	if (setenv("LC_ALL", "C", 1))
 		perror("Cannot set LC_ALL to C");
 
-	if (_restart)
-		_restart_dmeventd();
+	if (info)
+		return _info_dmeventd(argv[0], &fifos) ? EXIT_SUCCESS : EXIT_FAILURE;
 
 #ifdef __linux__
 	_systemd_activation = _systemd_handover(&fifos);
 #endif
 
+	dm_log_with_errno_init(_libdm_log);
+
+	if (restart) {
+		dm_event_log_set(debug_level, 0);
+
+		if ((restart = _restart_dmeventd(&fifos)) < 2)
+			return restart ? EXIT_SUCCESS : EXIT_FAILURE;
+	}
+
 	if (!_foreground)
 		_daemonize();
 
-	if (_use_syslog)
+	if (use_syslog)
 		openlog("dmeventd", LOG_PID, LOG_DAEMON);
 
-	dm_event_log_set(_debug_level, _use_syslog);
-	dm_log_with_errno_init(_libdm_log);
+	dm_event_log_set(debug_level, use_syslog);
 
 	(void) dm_prepare_selinux_context(DMEVENTD_PIDFILE, S_IFREG);
 	if (dm_create_lockfile(DMEVENTD_PIDFILE) == 0)
@@ -2282,15 +2660,28 @@ int main(int argc, char *argv[])
 					break;
 				}
 			}
-		} else if (_exit_now == DM_SIGNALED_EXIT) {
-			_exit_now = DM_SCHEDULED_EXIT;
-			/*
-			 * When '_exit_now' is set, signal has been received,
-			 * but can not simply exit unless all
-			 * threads are done processing.
-			 */
-			log_info("dmeventd received break, scheduling exit.");
-		}
+		} else
+			switch (_exit_now) {
+			case DM_SIGNALED_EXIT:
+				_exit_now = DM_SCHEDULED_EXIT;
+				/*
+				 * When '_exit_now' is set, signal has been received,
+				 * but can not simply exit unless all
+				 * threads are done processing.
+				 */
+				log_info("dmeventd received break, scheduling exit.");
+				/* fall through */
+			case DM_SCHEDULED_EXIT:
+				/* While exit is scheduled, check for exit_on file */
+				DEBUGLOG("Checking exit on file \"%s\".", _exit_on);
+				if (_exit_on[0] && (access(_exit_on, F_OK) == 0)) {
+					log_info("dmeventd detected exit on file %s, unregistering all monitored devices.",
+						 _exit_on);
+					_unregister_all_threads();
+				}
+				break;
+			}
+
 		_process_request(&fifos);
 		_cleanup_unused_threads();
 	}
@@ -2300,11 +2691,11 @@ int main(int argc, char *argv[])
 	log_notice("dmeventd shutting down.");
 
 	if (fifos.client >= 0 && close(fifos.client))
-		log_sys_error("client close", fifos.client_path);
+		log_sys_debug("client close", fifos.client_path);
 	if (fifos.server >= 0 && close(fifos.server))
-		log_sys_error("server close", fifos.server_path);
+		log_sys_debug("server close", fifos.server_path);
 
-	if (_use_syslog)
+	if (use_syslog)
 		closelog();
 
 	_exit_dm_lib();
