@@ -106,26 +106,31 @@ int dm_get_status_raid(struct dm_pool *mem, const char *params,
 
 	/* Second field holds the device count */
 	msg_fields = "<#devs> ";
-	if (!(p = _skip_fields(params, 1)) || (sscanf(p, "%d", &i) != 1))
+	if (!(pp = _skip_fields(params, 1)) || (sscanf(pp, "%d", &i) != 1) || !(p = _skip_fields(pp, 1)))
 		goto_bad;
 
 	msg_fields = "";
 	if (!(s = dm_pool_zalloc(mem, sizeof(struct dm_status_raid))))
 		goto_bad;
 
-	if (!(s->raid_type = dm_pool_zalloc(mem, p - params)))
+	msg_fields = "<raid_type> <#devices> <health_chars> and <sync_ratio> ";
+	if (!(s->raid_type = dm_pool_strndup(mem, params, pp - params - 1)))
 		goto_bad; /* memory is freed when pool is destroyed */
 
-	if (!(s->dev_health = dm_pool_zalloc(mem, i + 1))) /* Space for health chars */
+	if (!(pp = _skip_fields(p, 1)))
 		goto_bad;
 
-	msg_fields = "<raid_type> <#devices> <health_chars> and <sync_ratio> ";
-	if (sscanf(params, "%s %u %s " FMTu64 "/" FMTu64,
-		   s->raid_type,
-		   &s->dev_count,
-		   s->dev_health,
-		   &s->insync_regions,
-		   &s->total_regions) != 5)
+	/* Raid target can actually report more than real number of legs in a case
+	 * raid legs have been removed during initial raid array resynchronization */
+	if (i > (pp - p - 1))
+		i = pp - p - 1;
+
+	if (!(s->dev_health = dm_pool_strndup(mem, p, i))) /* health chars */
+		goto_bad;
+	p = pp;
+
+	s->dev_count = i;
+	if (sscanf(p, FMTu64 "/" FMTu64, &s->insync_regions, &s->total_regions) != 2)
 		goto_bad;
 
 	/*
@@ -141,13 +146,13 @@ int dm_get_status_raid(struct dm_pool *mem, const char *params,
 	msg_fields = "<sync_action> and <mismatch_cnt> ";
 
 	/* Skip pre-1.5.0 params */
-	if (!(p = _skip_fields(params, 4)) || !(pp = _skip_fields(p, 1)))
+	if (!(pp = _skip_fields(params, 4)) || !(p = _skip_fields(pp, 1)))
 		goto_bad;
 
-	if (!(s->sync_action = dm_pool_zalloc(mem, pp - p)))
+	if (!(s->sync_action = dm_pool_strndup(mem, pp, p - pp - 1)))
 		goto_bad;
 
-	if (sscanf(p, "%s " FMTu64, s->sync_action, &s->mismatch_count) != 2)
+	if (sscanf(p, FMTu64, &s->mismatch_count) != 1)
 		goto_bad;
 
 	if (num_fields < 7)
@@ -166,23 +171,35 @@ int dm_get_status_raid(struct dm_pool *mem, const char *params,
 	if (sscanf(p, FMTu64, &s->data_offset) != 1)
 		goto bad;
 
+	/* <journal_char>  - 'A' - active write-through journal device.
+	 *                 - 'a' - active write-back journal device.
+	 *                 - 'D' - dead journal device.
+	 *                 - '-' - no journal device.
+	 */
+
 out:
 	*status = s;
 
-	if (s->insync_regions == s->total_regions) {
-		/* FIXME: kernel gives misleading info here
-		 * Trying to recognize a true state */
-		while (i-- > 0)
-			if (s->dev_health[i] == 'a')
-				a++; /* Count number of 'a' */
+	while (i-- > 0)
+		if (s->dev_health[i] == 'a')
+			a++; /* Count number of 'a' */
 
-		if (a && a < s->dev_count) {
-			/* SOME legs are in 'a' */
-			if (!strcasecmp(s->sync_action, "recover")
-			    || !strcasecmp(s->sync_action, "idle"))
-				/* Kernel may possibly start some action
-				 * in near-by future, do not report 100% */
-				s->insync_regions--;
+	if (a) {
+		if ((a < s->dev_count) &&  /* SOME legs are in 'a' */
+		    /* FIXME: kernel gives misleading info here
+		     * Trying to recognize a true state */
+		    (s->insync_regions == s->total_regions) &&
+		    (!strcasecmp(s->sync_action, "recover") ||
+		     !strcasecmp(s->sync_action, "idle"))) {
+			/* Kernel may possibly start some action
+			 * in near-by future, do not report 100% */
+			s->insync_regions--;
+		}
+		if ((a == s->dev_count) && /* all legs are in 'a' */
+		    (!strcasecmp(s->sync_action, "resync") ||
+		     !strcasecmp(s->sync_action, "idle"))) {
+			/* Mark 1st. leg in sync */
+			s->dev_health[0] = 'A';
 		}
 	}
 
@@ -318,11 +335,19 @@ int dm_get_status_cache(struct dm_pool *mem, const char *params,
 
 	/* Read in policy args */
 	pp = p;
-	if (!(p = _skip_fields(p, 1)) ||
-	    !(s->policy_name = dm_pool_zalloc(mem, (p - pp))))
-		goto bad;
-	if (sscanf(pp, "%s %d", s->policy_name, &s->policy_argc) != 2)
+	if (!(p = _skip_fields(p, 1)))
+		goto_bad;
+
+	i = p - pp;
+	if ((i < 1) ||
+	    !(s->policy_name = dm_pool_zalloc(mem, i)))
+		goto_bad;
+
+	dm_strncpy(s->policy_name, pp, i);
+
+	if (sscanf(p, "%d", &s->policy_argc) != 1)
 		goto bad;
+
 	if (s->policy_argc &&
 	    (!(s->policy_argv = dm_pool_zalloc(mem, sizeof(char *) * s->policy_argc)) ||
 	     !(p = _skip_fields(p, 1)) ||
@@ -384,12 +409,12 @@ int dm_get_status_integrity(struct dm_pool *mem, const char *params,
 			     struct dm_status_integrity **status)
 {
 	struct dm_status_integrity *s;
-	char recalc_str[16] = "\0";
+	char recalc_str[16] = { 0 };
 
 	if (!(s = dm_pool_zalloc(mem, sizeof(*s))))
 		return_0;
 
-	if (sscanf(params, "%llu %llu %s",
+	if (sscanf(params, "%llu %llu %15s",
 		   (unsigned long long *)&s->number_of_mismatches,
 		   (unsigned long long *)&s->provided_data_sectors,
 		   recalc_str) != 3) {
@@ -511,6 +536,21 @@ int dm_get_status_thin(struct dm_pool *mem, const char *params,
 	return 1;
 }
 
+static dm_status_mirror_health_t _get_health(char c)
+{
+	switch (c) {
+	case 'A': return DM_STATUS_MIRROR_ALIVE;
+	case 'F': return DM_STATUS_MIRROR_FLUSH_FAILED;
+	case 'D': return DM_STATUS_MIRROR_WRITE_FAILED;
+	case 'S': return DM_STATUS_MIRROR_SYNC_FAILED;
+	case 'R': return DM_STATUS_MIRROR_READ_FAILED;
+	default:
+		log_warn("WARNING: Unknown mirror health status char: %c", c);
+		/* fall through */
+	case 'U': return DM_STATUS_MIRROR_UNCLASSIFIED;
+	}
+}
+
 /*
  * dm core parms:	     0 409600 mirror
  * Mirror core parms:	     2 253:4 253:5 400/400
@@ -539,7 +579,7 @@ int dm_get_status_mirror(struct dm_pool *mem, const char *params,
 	pos += used;
 
 	if (num_devs > DM_MIRROR_MAX_IMAGES) {
-		log_error(INTERNAL_ERROR "More then " DM_TO_STRING(DM_MIRROR_MAX_IMAGES)
+		log_error(INTERNAL_ERROR "More than " DM_TO_STRING(DM_MIRROR_MAX_IMAGES)
 			  " reported in mirror status.");
 		goto out;
 	}
@@ -564,7 +604,7 @@ int dm_get_status_mirror(struct dm_pool *mem, const char *params,
 	pos += used;
 
 	for (i = 0; i < num_devs ; ++i)
-		s->devs[i].health = pos[i];
+		s->devs[i].health = _get_health(pos[i]);
 
 	if (!(pos = _skip_fields(pos, argc)))
 		goto_out;
@@ -609,7 +649,7 @@ int dm_get_status_mirror(struct dm_pool *mem, const char *params,
 					goto_out;
 
 			for (i = 0; i < s->log_count; ++i)
-				s->logs[i].health = pos[i];
+				s->logs[i].health = _get_health(pos[i]);
 		}
 	}
 
