@@ -44,8 +44,8 @@ struct _dl_setup_baton {
 static int _da_setup(struct disk_locn *da, void *baton)
 {
 	struct _dl_setup_baton *p = baton;
-	p->pvh_dlocn_xl->offset = xlate64(da->offset);
-	p->pvh_dlocn_xl->size = xlate64(da->size);
+	p->pvh_dlocn_xl->offset = htole64(da->offset);
+	p->pvh_dlocn_xl->size = htole64(da->size);
 	p->pvh_dlocn_xl++;
 	return 1;
 }
@@ -63,8 +63,8 @@ static int _mda_setup(struct metadata_area *mda, void *baton)
 	if (mdac->area.dev != p->dev)
 		return 1;
 
-	p->pvh_dlocn_xl->offset = xlate64(mdac->area.start);
-	p->pvh_dlocn_xl->size = xlate64(mdac->area.size);
+	p->pvh_dlocn_xl->offset = htole64(mdac->area.start);
+	p->pvh_dlocn_xl->size = htole64(mdac->area.size);
 	p->pvh_dlocn_xl++;
 
 	return 1;
@@ -74,8 +74,8 @@ static int _dl_null_termination(void *baton)
 {
 	struct _dl_setup_baton *p = baton;
 
-	p->pvh_dlocn_xl->offset = xlate64(UINT64_C(0));
-	p->pvh_dlocn_xl->size = xlate64(UINT64_C(0));
+	p->pvh_dlocn_xl->offset = htole64(UINT64_C(0));
+	p->pvh_dlocn_xl->size = htole64(UINT64_C(0));
 	p->pvh_dlocn_xl++;
 
 	return 1;
@@ -99,9 +99,9 @@ static int _text_write(struct label *label, void *buf)
 
 	memcpy(lh->type, LVM2_LABEL, sizeof(lh->type));
 
-	pvhdr = (struct pv_header *) ((char *) buf + xlate32(lh->offset_xl));
+	pvhdr = (struct pv_header *) ((char *) buf + htole32(lh->offset_xl));
 	info = (struct lvmcache_info *) label->info;
-	pvhdr->device_size_xl = xlate64(lvmcache_device_size(info));
+	pvhdr->device_size_xl = htole64(lvmcache_device_size(info));
 	memcpy(pvhdr->pv_uuid, &lvmcache_device(info)->pvid, sizeof(struct id));
 	if (!id_write_format((const struct id *)pvhdr->pv_uuid, buffer,
 			     sizeof(buffer))) {
@@ -124,8 +124,8 @@ static int _text_write(struct label *label, void *buf)
 	 * PV header extension
 	 */
 	pvhdr_ext = (struct pv_header_extension *) ((char *) baton.pvh_dlocn_xl);
-	pvhdr_ext->version = xlate32(PV_HEADER_EXTENSION_VSN);
-	pvhdr_ext->flags = xlate32(lvmcache_ext_flags(info));
+	pvhdr_ext->version = htole32(PV_HEADER_EXTENSION_VSN);
+	pvhdr_ext->flags = htole32(lvmcache_ext_flags(info));
 
 	/* List of bootloader area locations */
 	baton.pvh_dlocn_xl = &pvhdr_ext->bootloader_areas_xl[0];
@@ -133,20 +133,20 @@ static int _text_write(struct label *label, void *buf)
 	_dl_null_termination(&baton);
 
 	/* Create debug message with ba, da and mda locations */
-	ba1 = (xlate64(pvhdr_ext->bootloader_areas_xl[0].offset) ||
-	       xlate64(pvhdr_ext->bootloader_areas_xl[0].size)) ? 0 : -1;
+	ba1 = (htole64(pvhdr_ext->bootloader_areas_xl[0].offset) ||
+	       htole64(pvhdr_ext->bootloader_areas_xl[0].size)) ? 0 : -1;
 
-	da1 = (xlate64(pvhdr->disk_areas_xl[0].offset) ||
-	       xlate64(pvhdr->disk_areas_xl[0].size)) ? 0 : -1;
+	da1 = (htole64(pvhdr->disk_areas_xl[0].offset) ||
+	       htole64(pvhdr->disk_areas_xl[0].size)) ? 0 : -1;
 
 	mda1 = da1 + 2;
 	mda2 = mda1 + 1;
 	
-	if (!xlate64(pvhdr->disk_areas_xl[mda1].offset) &&
-	    !xlate64(pvhdr->disk_areas_xl[mda1].size))
+	if (!htole64(pvhdr->disk_areas_xl[mda1].offset) &&
+	    !htole64(pvhdr->disk_areas_xl[mda1].size))
 		mda1 = mda2 = 0;
-	else if (!xlate64(pvhdr->disk_areas_xl[mda2].offset) &&
-		 !xlate64(pvhdr->disk_areas_xl[mda2].size))
+	else if (!htole64(pvhdr->disk_areas_xl[mda2].offset) &&
+		 !htole64(pvhdr->disk_areas_xl[mda2].size))
 		mda2 = 0;
 
 	log_debug_metadata("%s: Preparing PV label header %s size " FMTu64 " with"
@@ -157,31 +157,31 @@ static int _text_write(struct label *label, void *buf)
 			   dev_name(lvmcache_device(info)), buffer, lvmcache_device_size(info),
 			   (ba1 > -1) ? " ba1 (" : "",
 			   (ba1 > -1) ? 1 : 0,
-			   (ba1 > -1) ? xlate64(pvhdr_ext->bootloader_areas_xl[ba1].offset) >> SECTOR_SHIFT : 0,
+			   (ba1 > -1) ? htole64(pvhdr_ext->bootloader_areas_xl[ba1].offset) >> SECTOR_SHIFT : 0,
 			   (ba1 > -1) ? "s, " : "",
 			   (ba1 > -1) ? 1 : 0,
-			   (ba1 > -1) ? xlate64(pvhdr_ext->bootloader_areas_xl[ba1].size) >> SECTOR_SHIFT : 0,
+			   (ba1 > -1) ? htole64(pvhdr_ext->bootloader_areas_xl[ba1].size) >> SECTOR_SHIFT : 0,
 			   (ba1 > -1) ? "s)" : "",
 			   (da1 > -1) ? " da1 (" : "",
 			   (da1 > -1) ? 1 : 0,
-			   (da1 > -1) ? xlate64(pvhdr->disk_areas_xl[da1].offset) >> SECTOR_SHIFT : 0,
+			   (da1 > -1) ? htole64(pvhdr->disk_areas_xl[da1].offset) >> SECTOR_SHIFT : 0,
 			   (da1 > -1) ? "s, " : "",
 			   (da1 > -1) ? 1 : 0,
-			   (da1 > -1) ? xlate64(pvhdr->disk_areas_xl[da1].size) >> SECTOR_SHIFT : 0,
+			   (da1 > -1) ? htole64(pvhdr->disk_areas_xl[da1].size) >> SECTOR_SHIFT : 0,
 			   (da1 > -1) ? "s)" : "",
 			   mda1 ? " mda1 (" : "",
 			   mda1 ? 1 : 0,
-			   mda1 ? xlate64(pvhdr->disk_areas_xl[mda1].offset) >> SECTOR_SHIFT : 0,
+			   mda1 ? htole64(pvhdr->disk_areas_xl[mda1].offset) >> SECTOR_SHIFT : 0,
 			   mda1 ? "s, " : "",
 			   mda1 ? 1 : 0,
-			   mda1 ? xlate64(pvhdr->disk_areas_xl[mda1].size) >> SECTOR_SHIFT : 0,
+			   mda1 ? htole64(pvhdr->disk_areas_xl[mda1].size) >> SECTOR_SHIFT : 0,
 			   mda1 ? "s)" : "",
 			   mda2 ? " mda2 (" : "",
 			   mda2 ? 1 : 0,
-			   mda2 ? xlate64(pvhdr->disk_areas_xl[mda2].offset) >> SECTOR_SHIFT : 0,
+			   mda2 ? htole64(pvhdr->disk_areas_xl[mda2].offset) >> SECTOR_SHIFT : 0,
 			   mda2 ? "s, " : "",
 			   mda2 ? 1 : 0,
-			   mda2 ? xlate64(pvhdr->disk_areas_xl[mda2].size) >> SECTOR_SHIFT : 0,
+			   mda2 ? htole64(pvhdr->disk_areas_xl[mda2].size) >> SECTOR_SHIFT : 0,
 			   mda2 ? "s)" : "");
 
 	if (da1 < 0) {
@@ -332,7 +332,7 @@ static int _read_mda_header_and_metadata(const struct format_type *fmt,
  retry:
 
 	if (!(mdah = raw_read_mda_header(fmt, &mdac->area, (mda->mda_num == 1), 0, bad_fields))) {
-		log_warn("WARNING: bad metadata header on %s at %llu.",
+		log_warn("WARNING: Bad metadata header on %s at %llu.",
 			 dev_name(mdac->area.dev),
 			 (unsigned long long)mdac->area.start);
 		mda->header_start = mdac->area.start;
@@ -344,7 +344,7 @@ static int _read_mda_header_and_metadata(const struct format_type *fmt,
 
 	mda_set_ignored(mda, rlocn_is_ignored(mdah->raw_locns));
 
-	if (mda_is_ignored(mda)) {
+	if (_mda_is_ignored(mda)) {
 		log_debug_metadata("Ignoring mda on device %s at offset " FMTu64,
 				   dev_name(mdac->area.dev),
 				   mdac->area.start);
@@ -383,13 +383,13 @@ static int _read_mda_header_and_metadata(const struct format_type *fmt,
 		 * somehow between the two reads.)
 		 */
 		if (!retries) {
-			log_print("Retrying metadata scan.");
+			log_print_unless_silent("Retrying metadata scan.");
 			retries++;
 			dev_invalidate(mdac->area.dev);
 			goto retry;
 		}
 
-		log_warn("WARNING: bad metadata text on %s in mda%d",
+		log_warn("WARNING: Bad metadata text on %s in mda%d.",
 			 dev_name(mdac->area.dev), mda->mda_num);
 		*bad_fields |= BAD_MDA_TEXT;
 		return 0;
@@ -409,6 +409,8 @@ static int _text_read(struct cmd_context *cmd, struct labeller *labeller, struct
 		      uint64_t label_sector, int *is_duplicate)
 {
 	struct lvmcache_vgsummary vgsummary;
+	char pvid[ID_LEN + 1] __attribute__((aligned(8))) = { 0 };
+	char vgid[ID_LEN + 1] __attribute__((aligned(8))) = { 0 };
 	struct lvmcache_info *info;
 	const struct format_type *fmt = labeller->fmt;
 	struct label_header *lh = (struct label_header *) label_buf;
@@ -429,7 +431,10 @@ static int _text_read(struct cmd_context *cmd, struct labeller *labeller, struct
 	/*
 	 * PV header base
 	 */
-	pvhdr = (struct pv_header *) ((char *) label_buf + xlate32(lh->offset_xl));
+	pvhdr = (struct pv_header *) ((char *) label_buf + htole32(lh->offset_xl));
+
+	memcpy(pvid, &pvhdr->pv_uuid, ID_LEN);
+	strncpy(vgid, FMT_TEXT_ORPHAN_VG_NAME, ID_LEN);
 
 	/*
 	 * FIXME: stop adding the device to lvmcache initially as an orphan
@@ -445,12 +450,12 @@ static int _text_read(struct cmd_context *cmd, struct labeller *labeller, struct
 	 *
 	 * Other reasons for lvmcache_add to return NULL are internal errors.
 	 */
-	if (!(info = lvmcache_add(cmd, labeller, (char *)pvhdr->pv_uuid, dev, label_sector,
-				  FMT_TEXT_ORPHAN_VG_NAME,
-				  FMT_TEXT_ORPHAN_VG_NAME, 0, is_duplicate)))
+	if (!(info = lvmcache_add(cmd, labeller, pvid, dev, label_sector,
+				  vgid,
+				  vgid, 0, is_duplicate)))
 		return_0;
 
-	lvmcache_set_device_size(info, xlate64(pvhdr->device_size_xl));
+	lvmcache_set_device_size(info, htole64(pvhdr->device_size_xl));
 
 	lvmcache_del_das(info);
 	lvmcache_del_mdas(info);
@@ -458,20 +463,20 @@ static int _text_read(struct cmd_context *cmd, struct labeller *labeller, struct
 
 	/* Data areas holding the PEs */
 	dlocn_xl = pvhdr->disk_areas_xl;
-	while ((offset = xlate64(dlocn_xl->offset))) {
-		lvmcache_add_da(info, offset, xlate64(dlocn_xl->size));
+	while ((offset = htole64(dlocn_xl->offset))) {
+		lvmcache_add_da(info, offset, htole64(dlocn_xl->size));
 		dlocn_xl++;
 	}
 
 	dlocn_xl++;
 
 	/* Metadata areas */
-	while ((offset = xlate64(dlocn_xl->offset))) {
+	while ((offset = htole64(dlocn_xl->offset))) {
 
 		/*
 		 * This just calls add_mda() above, replacing info with info->mdas.
 		 */
-		lvmcache_add_mda(info, dev, offset, xlate64(dlocn_xl->size), 0, &mda);
+		lvmcache_add_mda(info, dev, offset, htole64(dlocn_xl->size), 0, &mda);
 
 		dlocn_xl++;
 		mda_count++;
@@ -492,22 +497,23 @@ static int _text_read(struct cmd_context *cmd, struct labeller *labeller, struct
 	 * PV header extension
 	 */
 	pvhdr_ext = (struct pv_header_extension *) ((char *) dlocn_xl);
-	if (!(ext_version = xlate32(pvhdr_ext->version)))
+	if (!(ext_version = htole32(pvhdr_ext->version)))
 		goto scan_mdas;
 
-	log_debug_metadata("%s: PV header extension version " FMTu32 " found",
-			   dev_name(dev), ext_version);
+	if (ext_version != PV_HEADER_EXTENSION_VSN)
+		log_debug_metadata("Found pv_header_extension version " FMTu32 " on %s",
+				   ext_version, dev_name(dev));
 
 	/* Extension version */
-	lvmcache_set_ext_version(info, xlate32(pvhdr_ext->version));
+	lvmcache_set_ext_version(info, htole32(pvhdr_ext->version));
 
 	/* Extension flags */
-	lvmcache_set_ext_flags(info, xlate32(pvhdr_ext->flags));
+	lvmcache_set_ext_flags(info, htole32(pvhdr_ext->flags));
 
 	/* Bootloader areas */
 	dlocn_xl = pvhdr_ext->bootloader_areas_xl;
-	while ((offset = xlate64(dlocn_xl->offset))) {
-		lvmcache_add_ba(info, offset, xlate64(dlocn_xl->size));
+	while ((offset = htole64(dlocn_xl->offset))) {
+		lvmcache_add_ba(info, offset, htole64(dlocn_xl->size));
 		dlocn_xl++;
 	}
 
@@ -557,7 +563,7 @@ static int _text_read(struct cmd_context *cmd, struct labeller *labeller, struct
 				bad_mda_count++;
 			} else {
 				/* The normal success path */
-				log_debug("Scanned %s mda1 seqno %u", dev_name(dev), vgsummary.seqno);
+				log_debug("Found metadata seqno %u in mda1 on %s", vgsummary.seqno, dev_name(dev));
 				good_mda_count++;
 			}
 		}
@@ -568,8 +574,8 @@ static int _text_read(struct cmd_context *cmd, struct labeller *labeller, struct
 			 * used by vg_read/vg_write, but keep track of it in
 			 * lvmcache for repair.
 			 */
-			log_warn("WARNING: scanning %s mda1 failed to read metadata summary.", dev_name(dev));
-			log_warn("WARNING: repair VG metadata on %s with vgck --updatemetadata.", dev_name(dev));
+			log_warn("WARNING: Scanning %s mda1 failed to read metadata summary.", dev_name(dev));
+			log_warn("WARNING: Repair VG metadata on %s with vgck --updatemetadata.", dev_name(dev));
 
 			dm_list_del(&mda1->list);
 			mda1->bad_fields = bad_fields;
@@ -607,7 +613,7 @@ static int _text_read(struct cmd_context *cmd, struct labeller *labeller, struct
 				bad_mda_count++;
 			} else {
 				/* The normal success path */
-				log_debug("Scanned %s mda2 seqno %u", dev_name(dev), vgsummary.seqno);
+				log_debug("Found metadata seqno %u in mda2 on %s", vgsummary.seqno, dev_name(dev));
 				good_mda_count++;
 			}
 		}
@@ -618,8 +624,8 @@ static int _text_read(struct cmd_context *cmd, struct labeller *labeller, struct
 			 * used by vg_read/vg_write, but keep track of it in
 			 * lvmcache for repair.
 			 */
-			log_warn("WARNING: scanning %s mda2 failed to read metadata summary.", dev_name(dev));
-			log_warn("WARNING: repair VG metadata on %s with vgck --updatemetadata.", dev_name(dev));
+			log_warn("WARNING: Scanning %s mda2 failed to read metadata summary.", dev_name(dev));
+			log_warn("WARNING: Repair VG metadata on %s with vgck --updatemetadata.", dev_name(dev));
 
 			dm_list_del(&mda2->list);
 			mda2->bad_fields = bad_fields;
@@ -654,7 +660,7 @@ static void _fmt_text_destroy(struct labeller *l)
 	free(l);
 }
 
-struct label_ops _text_ops = {
+static const struct label_ops _text_ops = {
 	.can_handle = _text_can_handle,
 	.write = _text_write,
 	.read = _text_read,
