@@ -222,8 +222,8 @@ int update_cache_pool_params(struct cmd_context *cmd,
 		}
 		if (*chunk_size < min_chunk_size) {
 			/*
-			 * When using more then 'standard' default,
-			 * keep user informed he might be using things in untintended direction
+			 * When using more than 'standard' default,
+			 * keep user informed he might be using things in unintended direction
 			 */
 			log_print_unless_silent("Using %s chunk size instead of default %s, "
 						"so cache pool has less than " FMTu64 " chunks.",
@@ -356,17 +356,10 @@ int validate_lv_cache_create_origin(const struct logical_volume *origin_lv)
 		return 0;
 	}
 
-	/*
-	 * Only linear, striped or raid supported.
-	 * FIXME Tidy up all these type restrictions.
-	 */
 	if (lv_is_cache_type(origin_lv) ||
 	    lv_is_mirror_type(origin_lv) ||
-	    lv_is_thin_volume(origin_lv) || lv_is_thin_pool_metadata(origin_lv) ||
 	    lv_is_merging_origin(origin_lv) ||
-	    lv_is_cow(origin_lv) || lv_is_merging_cow(origin_lv) ||
-	    /* TODO: think about enabling caching of a single thin volume */
-	    (lv_is_virtual(origin_lv) && !lv_is_vdo(origin_lv))) {
+	    lv_is_cow(origin_lv) || lv_is_merging_cow(origin_lv)) {
 		log_error("Cache is not supported with %s segment type of the original logical volume %s.",
 			  lvseg_name(first_seg(origin_lv)), display_lvname(origin_lv));
 		return 0;
@@ -427,8 +420,8 @@ struct logical_volume *lv_cache_create(struct logical_volume *pool_lv,
 
 	if (!(segtype = get_segtype_from_string(cmd, SEG_TYPE_NAME_CACHE)))
 		return_NULL;
-
-	if (!insert_layer_for_lv(cmd, cache_lv, CACHE, "_corig"))
+	/* coverity[format_string_injection] lv name is already validated */
+	if (!insert_layer_for_lv(cmd, cache_lv, 0, "_corig"))
 		return_NULL;
 
 	seg = first_seg(cache_lv);
@@ -475,6 +468,7 @@ int lv_cache_wait_for_clean(struct logical_volume *cache_lv, int *is_clean)
 			if (cache_seg->cleaner_policy) {
 				cache_seg->cleaner_policy = 0;
 				/* Restore normal table */
+				sigint_clear();
 				if (!lv_update_and_reload_origin(cache_lv))
 					stack;
 			}
@@ -486,7 +480,7 @@ int lv_cache_wait_for_clean(struct logical_volume *cache_lv, int *is_clean)
 
 		if (status->cache->fail) {
 			dm_pool_destroy(status->mem);
-			log_warn("WARNING: Skippping flush for failed cache %s.",
+			log_warn("WARNING: Skipping flush for failed cache %s.",
 				 display_lvname(cache_lv));
 			return 1;
 		}
@@ -565,6 +559,7 @@ int lv_cache_remove(struct logical_volume *cache_lv)
 	struct id *data_id, *metadata_id;
 	uint64_t data_len, metadata_len;
 	cache_mode_t cache_mode;
+	int temp_activated = 0;
 	int is_clear;
 
 	if (!lv_is_cache(cache_lv)) {
@@ -579,11 +574,13 @@ int lv_cache_remove(struct logical_volume *cache_lv)
 		goto remove;  /* Already dropped */
 	}
 
-	/* Localy active volume is needed for writeback */
 	if (!lv_info(cache_lv->vg->cmd, cache_lv, 1, NULL, 0, 0)) {
-		/* Give up any remote locks */
-		if (!deactivate_lv_with_sub_lv(cache_lv))
-			return_0;
+
+		/*
+		 * LV is inactive.  When used in writeback, it will
+		 * need to be activated to write the cache content
+		 * back to the main LV before detaching it.
+		 */
 
 		cache_mode = (lv_is_cache_pool(cache_seg->pool_lv)) ?
 			first_seg(cache_seg->pool_lv)->cache_mode : cache_seg->cache_mode;
@@ -600,7 +597,6 @@ int lv_cache_remove(struct logical_volume *cache_lv)
 				return_0;
 			return 1;
 		default:
-			/* Otherwise localy activate volume to sync dirty blocks */
 			cache_lv->status |= LV_TEMPORARY;
 			if (!activate_lv(cache_lv->vg->cmd, cache_lv) ||
 			    !lv_is_active(cache_lv)) {
@@ -608,6 +604,7 @@ int lv_cache_remove(struct logical_volume *cache_lv)
 				return 0;
 			}
 			cache_lv->status &= ~LV_TEMPORARY;
+			temp_activated = 1;
 		}
 	}
 
@@ -626,8 +623,14 @@ int lv_cache_remove(struct logical_volume *cache_lv)
 	 * remove the cache_pool then without waiting for the flush to
 	 * complete.
 	 */
-	if (!lv_cache_wait_for_clean(cache_lv, &is_clear))
+	if (!lv_cache_wait_for_clean(cache_lv, &is_clear)) {
+		if (temp_activated && !deactivate_lv(cache_lv->vg->cmd, cache_lv))
+			stack;
 		return_0;
+	}
+
+	if (temp_activated && !deactivate_lv(cache_lv->vg->cmd, cache_lv))
+		log_warn("Failed to deactivate after cleaning cache.");
 
 	cache_pool_lv = cache_seg->pool_lv;
 	if (!detach_pool_lv(cache_seg))
@@ -699,7 +702,7 @@ remove:
 	if (!lv_remove(cache_lv)) /* Will use LV_PENDING_DELETE */
 		return_0;
 
-	/* CachePool or CacheVol is left inactivate for further manipulation */
+	/* CachePool or CacheVol is left inactive for further manipulation */
 
 	return 1;
 }
@@ -749,7 +752,7 @@ static const char *_get_default_cache_policy(struct cmd_context *cmd)
 static cache_metadata_format_t _get_default_cache_metadata_format(struct cmd_context *cmd)
 {
 	const struct segment_type *segtype = get_segtype_from_string(cmd, SEG_TYPE_NAME_CACHE);
-	unsigned attr;
+	unsigned attr = 0;
 	cache_metadata_format_t f;
 
 	if (!segtype ||
@@ -920,12 +923,17 @@ int cache_set_metadata_format(struct lv_segment *seg, cache_metadata_format_t fo
 			return 1; /* Format already selected in cache pool */
 
 		/* Check configurations and profiles */
-		format = find_config_tree_int(seg->lv->vg->cmd, allocation_cache_metadata_format_CFG,
-					      profile);
+		switch (find_config_tree_int(seg->lv->vg->cmd,
+					     allocation_cache_metadata_format_CFG,
+					     profile)) {
+		case 1:  format = CACHE_METADATA_FORMAT_1; break;
+		case 2:  format = CACHE_METADATA_FORMAT_2; break;
+		default: format = CACHE_METADATA_FORMAT_UNSELECTED; break;
+		}
 	}
 
 	/* See what is a 'best' available cache metadata format
-	 * when the specifed format is other then always existing CMFormat 1 */
+	 * when the specified format is other then always existing CMFormat 1 */
 	if (format != CACHE_METADATA_FORMAT_1) {
 		best = _get_default_cache_metadata_format(seg->lv->vg->cmd);
 
@@ -1167,8 +1175,8 @@ int cache_vol_set_params(struct cmd_context *cmd,
 		log_warn("WARNING: Data redundancy could be lost with writeback caching of raid logical volume!");
 
 	if (lv_is_thin_pool_data(cache_lv)) {
-		log_warn("WARNING: thin pool data will not be automatically extended when cached.");
-		log_warn("WARNING: manual splitcache is required before extending thin pool data.");
+		log_warn("WARNING: Thin pool data will not be automatically extended when cached.");
+		log_warn("WARNING: Manual splitcache is required before extending thin pool data.");
 	}
 
 	cache_seg->chunk_size = chunk_size;
@@ -1229,6 +1237,7 @@ int cache_set_params(struct lv_segment *seg,
 		if (!pool_seg->chunk_size &&
 		    /* TODO: some calc_policy solution for cache ? */
 		    !recalculate_pool_chunk_size_with_dev_hints(pool_seg->lv,
+								seg_lv(pool_seg, 0),
 								THIN_CHUNK_SIZE_CALC_METHOD_GENERIC))
 			return_0;
 	}
@@ -1248,6 +1257,7 @@ int cache_set_params(struct lv_segment *seg,
 int wipe_cache_pool(struct logical_volume *cache_pool_lv)
 {
 	int r;
+	struct logical_volume *cache_data_lv;
 
 	/* Only unused cache-pool could be activated and wiped */
 	if (lv_is_used_cache_pool(cache_pool_lv) || lv_is_cache_vol(cache_pool_lv)) {
@@ -1256,6 +1266,16 @@ int wipe_cache_pool(struct logical_volume *cache_pool_lv)
 		return 0;
 	}
 
+	cache_data_lv = (lv_is_cache_pool(cache_pool_lv)) ?
+		seg_lv(first_seg(cache_pool_lv), 0) : cache_pool_lv;
+
+	if (cache_data_lv && seg_cannot_be_zeroed(first_seg(cache_data_lv))) {
+		log_debug("Skipping wipe of %s volume with %s segtype.",
+			  display_lvname(cache_data_lv),
+			  first_seg(cache_data_lv)->segtype->name);
+		return 1;
+	}
+
 	cache_pool_lv->status |= LV_TEMPORARY;
 	if (!activate_lv(cache_pool_lv->vg->cmd, cache_pool_lv)) {
 		log_error("Aborting. Failed to activate cache pool %s.",
