@@ -212,7 +212,7 @@ prepare_dmeventd() {
 	test "${LVM_VALGRIND_DMEVENTD:-0}" -eq 0 || run_valgrind="run_valgrind"
 	echo -n "## preparing dmeventd..."
 #	LVM_LOG_FILE_EPOCH=DMEVENTD $run_valgrind dmeventd -fddddl "$@" 2>&1 &
-	LVM_LOG_FILE_EPOCH=DMEVENTD $run_valgrind dmeventd -fddddl "$@" >debug.log_DMEVENTD_out 2>&1 &
+	LVM_LOG_FILE_EPOCH=DMEVENTD $run_valgrind dmeventd -fddddl -g 2 "$@" >debug.log_DMEVENTD_out 2>&1 &
 	echo $! > LOCAL_DMEVENTD
 
 	# FIXME wait for pipe in /var/run instead
@@ -518,7 +518,9 @@ kill_sleep_kill_() {
 	if test -s "$pidfile" ; then
 		pid=$(< "$pidfile")
 		rm -f "$pidfile"
-		test "$pidfile" = "LOCAL_LVMDBUSD" && killall -9 lvmdbusd || true
+		if test "$pidfile" = "LOCAL_LVMDBUSD"; then
+			killall -9 lvmdbusd || true
+		fi
 		kill -TERM "$pid" 2>/dev/null || return 0
 		for i in {0..10} ; do
 			ps "$pid" >/dev/null || return 0
@@ -1635,6 +1637,7 @@ dmeventd/executable = "$abs_top_builddir/test/lib/dmeventd"
 activation/udev_rules = 1
 activation/udev_sync = 1
 global/fsadm_executable = "$abs_top_builddir/test/lib/fsadm"
+global/lvresize_fs_helper_executable = "$abs_top_builddir/test/lib/lvresize_fs_helper"
 global/library_dir = "$TESTDIR/lib"
 global/locking_dir = "$TESTDIR/var/lock/lvm"
 EOF
@@ -1989,6 +1992,22 @@ have_raid() {
 	esac
 }
 
+have_raid_resizable() {
+	# Check if the kernel can resize raid volume without killing leg.
+	# MD core has had a bug in handling read-ahead requests.
+	# This results in marking raid with suspended leg as failed.
+	# Bug was fixed from kernel 6.16
+	# Fixing kernel commit: 9f346f7d4ea73692b82f5102ca8698e4040469ea
+	# Patch was backported to most 6.15 releases.
+	# Note: Fixing test for this failure would likely require full raid
+	# array resync, so it's better to skip the test for affected kernels.
+	case "$(uname -r)" in
+	  6.1[34]*) return 1 ;;
+	  5.14.0-611.el9.*) return 1 ;; # Kernel is missing fixing commit!
+	  6.12.0-124.el10.*) return 1 ;; # -- '' --
+	esac
+}
+
 have_raid4 () {
 	local r=0
 
