@@ -40,7 +40,7 @@ struct settings {
 	uint64_t device_size;      /* bytes */
 	uint64_t data_offset;      /* bytes, start of data (pe_start) */
 	uint32_t seqno;
-	struct id pvid;
+	struct id pv_id;
 
 	int mda_num;               /* 1 or 2 for first or second mda */
 	char *backup_file;
@@ -284,7 +284,7 @@ static int _text_buf_parsable(char *text_buf, uint64_t text_size)
 #define MAX_LINE_CHECK 128
 
 #define MAX_DESC 1024
-char desc_line[MAX_DESC];
+static char _desc_line[MAX_DESC];
 
 static void _copy_line(char *in, char *out, int *len, int linesize)
 {
@@ -379,14 +379,17 @@ static bool _read_bytes(struct device *dev, struct devicefile *def, uint64_t sta
 	if (dev)
 		return dev_read_bytes(dev, start, len, data);
 
+	if (!def)
+		return false;
+
 	off = lseek(def->fd, start, SEEK_SET);
-	if (off != start)
+	if (off != (off_t)start)
 		return false;
 
 	rv = read(def->fd, data, len);
 	if (rv < 0)
 		return false;
-	if (rv != len)
+	if ((size_t)rv != len)
 		return false;
 	return true;
 }
@@ -398,7 +401,7 @@ static int _dump_all_text(struct cmd_context *cmd, struct settings *set, const c
 			  int mda_num, uint64_t mda_offset, uint64_t mda_size, char *buf)
 {
 	FILE *fp = NULL;
-	char line[MAX_LINE_CHECK];
+	char line[MAX_LINE_CHECK + 3];
 	char vgname[NAME_LEN+1];
 	char id_str[ID_STR_SIZE];
 	char id_first[ID_STR_SIZE];
@@ -415,7 +418,7 @@ static int _dump_all_text(struct cmd_context *cmd, struct settings *set, const c
 	int multiple_vgs = 0;
 	int bad_end;
 	int vgnamelen;
-	int count;
+	unsigned count;
 	int len;
 
 	if (tofile) {
@@ -593,25 +596,25 @@ static int _dump_all_text(struct cmd_context *cmd, struct settings *set, const c
 		 */
 
 		if (!_text_buf_parsable(text_buf, text_size))
-			log_warn("WARNING: parse error for metadata at %llu", (unsigned long long)(mda_offset + buf_off));
+			log_warn("WARNING: Parse error for metadata at %llu.", (unsigned long long)(mda_offset + buf_off));
 		if (bad_end)
-			log_warn("WARNING: unexpected terminating bytes for metadata at %llu", (unsigned long long)(mda_offset + buf_off));
+			log_warn("WARNING: Unexpected terminating bytes for metadata at %llu.", (unsigned long long)(mda_offset + buf_off));
 
 		if (arg_is_set(cmd, verbose_ARG)) {
 			char *str1, *str2;
 			if ((str1 = strstr(text_buf, "description = "))) {
-				memset(desc_line, 0, sizeof(desc_line));
-				_copy_line(str1, desc_line, &len, sizeof(desc_line)-1);
-				if ((p = strchr(desc_line, '\n')))
+				memset(_desc_line, 0, sizeof(_desc_line));
+				_copy_line(str1, _desc_line, &len, sizeof(_desc_line)-1);
+				if ((p = strchr(_desc_line, '\n')))
 					*p = '\0';
-				log_print("%s", desc_line);
+				log_print("%s", _desc_line);
 			}
 			if (str1 && (str2 = strstr(str1, "creation_time = "))) {
-				memset(desc_line, 0, sizeof(desc_line));
-				_copy_line(str2, desc_line, &len, sizeof(desc_line)-1);
-				if ((p = strchr(desc_line, '\n')))
+				memset(_desc_line, 0, sizeof(_desc_line));
+				_copy_line(str2, _desc_line, &len, sizeof(_desc_line)-1);
+				if ((p = strchr(_desc_line, '\n')))
 					*p = '\0';
-				log_print("%s\n", desc_line);
+				log_print("%s\n", _desc_line);
 			}
 		}
 
@@ -633,7 +636,7 @@ static int _dump_all_text(struct cmd_context *cmd, struct settings *set, const c
 	}
 
 	if (multiple_vgs)
-		log_warn("WARNING: metadata from multiple VGs was found.");
+		log_warn("WARNING: Metadata from multiple VGs was found.");
 
 	if (fp) {
 		if (fflush(fp))
@@ -660,7 +663,7 @@ static int _check_label_header(struct label_header *lh, uint64_t labelsector,
 		bad++;
 	}
 
-	if (xlate64(lh->sector_xl) != labelsector) {
+	if (htole64(lh->sector_xl) != labelsector) {
 		log_print("CHECK: label_header.sector expected %d", (int)labelsector);
 		bad++;
 	}
@@ -668,12 +671,12 @@ static int _check_label_header(struct label_header *lh, uint64_t labelsector,
 	crc = calc_crc(INITIAL_CRC, (uint8_t *)&lh->offset_xl,
 		       LABEL_SIZE - ((uint8_t *) &lh->offset_xl - (uint8_t *) lh));
 
-	if (crc != xlate32(lh->crc_xl)) {
+	if (crc != htole32(lh->crc_xl)) {
 		log_print("CHECK: label_header.crc expected 0x%x", crc);
 		bad++;
 	}
 
-	if (xlate32(lh->offset_xl) != 32) {
+	if (htole32(lh->offset_xl) != 32) {
 		log_print("CHECK: label_header.offset expected 32");
 		bad++;
 	}
@@ -724,7 +727,7 @@ static int _check_mda_header(struct mda_header *mh, int mda_num, uint64_t mda_of
 	crc = calc_crc(INITIAL_CRC, (uint8_t *)mh->magic,
 		       MDA_HEADER_SIZE - sizeof(mh->checksum_xl));
 
-	if (crc != xlate32(mh->checksum_xl)) {
+	if (crc != htole32(mh->checksum_xl)) {
 		log_print("CHECK: mda_header_%d.checksum expected 0x%x", mda_num, crc);
 		bad++;
 	}
@@ -735,17 +738,17 @@ static int _check_mda_header(struct mda_header *mh, int mda_num, uint64_t mda_of
 		bad++;
 	}
 
-	if (xlate32(mh->version) != FMTT_VERSION) {
+	if (htole32(mh->version) != FMTT_VERSION) {
 		log_print("CHECK: mda_header_%d.version expected %u", mda_num, FMTT_VERSION);
 		bad++;
 	}
 
-	if (xlate64(mh->start) != mda_offset) {
+	if (htole64(mh->start) != mda_offset) {
 		log_print("CHECK: mda_header_%d.start does not match pv_header.disk_locn.offset %llu", mda_num, (unsigned long long)mda_offset);
 		bad++;
 	}
 
-	if (xlate64(mh->size) != mda_size) {
+	if (htole64(mh->size) != mda_size) {
 		log_print("CHECK: mda_header_%d.size does not match pv_header.disk_locn.size %llu", mda_num, (unsigned long long)mda_size);
 		bad++;
 	}
@@ -779,15 +782,14 @@ static int _dump_raw_locn(struct device *dev, struct devicefile *def, int print_
 	uint64_t meta_offset, meta_size;
 	uint32_t meta_checksum;
 	uint32_t meta_flags;
-	int bad = 0;
 	int mn = mda_num; /* 1 or 2 */
 	int ri = rlocn_index; /* 0 or 1 */
 	int wrapped = 0;
 
-	meta_offset = xlate64(rlocn->offset);
-	meta_size = xlate64(rlocn->size);
-	meta_checksum = xlate32(rlocn->checksum);
-	meta_flags = xlate32(rlocn->flags);
+	meta_offset = htole64(rlocn->offset);
+	meta_size = htole64(rlocn->size);
+	meta_checksum = htole32(rlocn->checksum);
+	meta_flags = htole32(rlocn->flags);
 
 	if (meta_offset + meta_size > mda_size)
 		wrapped = 1;
@@ -834,8 +836,6 @@ static int _dump_raw_locn(struct device *dev, struct devicefile *def, int print_
 	if (!meta_offset)
 		return 1;
 
-	if (bad)
-		return 0;
 	return 1;
 }
 
@@ -849,7 +849,7 @@ static int _dump_meta_area(struct device *dev, struct devicefile *def, const cha
 	if (!tofile)
 		return_0;
 
-	if (!(meta_buf = zalloc(mda_size)))
+	if (!(meta_buf = zalloc(mda_size + 1)))
 		return_0;
 
 	if (!_read_bytes(dev, def, mda_offset, mda_size, meta_buf)) {
@@ -944,6 +944,7 @@ static int _dump_current_text(struct device *dev, struct devicefile *def,
 		}
 	}
 
+	meta_buf[meta_size] = 0;
 	crc = calc_crc(INITIAL_CRC, (uint8_t *)meta_buf, meta_size);
 	if (crc != meta_checksum) {
 		log_print("CHECK: metadata text at %llu crc does not match mda_header_%d.raw_locn[%d].checksum",
@@ -1014,7 +1015,7 @@ static int _dump_label_and_pv_header(struct cmd_context *cmd, uint64_t labelsect
 				     uint64_t *mda2_offset, uint64_t *mda2_size,
 				     int *mda_count_out)
 {
-	char buf[512];
+	char buf[512 + 1] = { 0 };
 	char str[256];
 	struct label_header *lh;
 	struct pv_header *pvh;
@@ -1024,7 +1025,6 @@ static int _dump_label_and_pv_header(struct cmd_context *cmd, uint64_t labelsect
 	uint64_t pvh_offset;    /* bytes */
 	uint64_t pvhe_offset;   /* bytes */
 	uint64_t dlocn_offset;  /* bytes */
-	uint64_t tmp;
 	int mda_count = 0;
 	int bad = 0;
 	int di;
@@ -1042,9 +1042,9 @@ static int _dump_label_and_pv_header(struct cmd_context *cmd, uint64_t labelsect
 	if (print_fields) {
 		log_print("label_header at %llu", (unsigned long long)lh_offset);
 		log_print("label_header.id %s", _chars_to_str(lh->id, str, 8, 256, "label_header.id"));
-		log_print("label_header.sector %llu", (unsigned long long)xlate64(lh->sector_xl));
-		log_print("label_header.crc 0x%x", xlate32(lh->crc_xl));
-		log_print("label_header.offset %u", xlate32(lh->offset_xl));
+		log_print("label_header.sector %llu", (unsigned long long)htole64(lh->sector_xl));
+		log_print("label_header.crc 0x%x", htole32(lh->crc_xl));
+		log_print("label_header.offset %u", htole32(lh->offset_xl));
 		log_print("label_header.type %s", _chars_to_str(lh->type, str, 8, 256, "label_header.type"));
 	}
 
@@ -1065,14 +1065,10 @@ static int _dump_label_and_pv_header(struct cmd_context *cmd, uint64_t labelsect
 	pvh = (struct pv_header *)(buf + 32);
 	pvh_offset = lh_offset + 32; /* from start of disk */
 
-	/* sanity check */
-	if ((void *)pvh != (void *)(buf + pvh_offset - lh_offset))
-		log_print("CHECK: problem with pv_header offset calculation");
-
 	if (print_fields) {
 		log_print("pv_header at %llu", (unsigned long long)pvh_offset);
 		log_print("pv_header.pv_uuid %s", _chars_to_str(pvh->pv_uuid, str, ID_LEN, 256, "pv_header.pv_uuid"));
-		log_print("pv_header.device_size %llu", (unsigned long long)xlate64(pvh->device_size_xl));
+		log_print("pv_header.device_size %llu", (unsigned long long)htole64(pvh->device_size_xl));
 	}
 
 	if (!_check_pv_header(pvh))
@@ -1087,18 +1083,14 @@ static int _dump_label_and_pv_header(struct cmd_context *cmd, uint64_t labelsect
 	dlocn = pvh->disk_areas_xl;
 	dlocn_offset = pvh_offset + 40; /* from start of disk */
 
-	/* sanity check */
-	if ((void *)dlocn != (void *)(buf + dlocn_offset - lh_offset))
-		log_print("CHECK: problem with pv_header.disk_locn[%d] offset calculation", di);
-
-	while ((tmp = xlate64(dlocn->offset))) {
+	while (htole64(dlocn->offset)) {
 		if (print_fields) {
 			log_print("pv_header.disk_locn[%d] at %llu # location of data area", di,
 				  (unsigned long long)dlocn_offset);
 			log_print("pv_header.disk_locn[%d].offset %llu", di,
-				  (unsigned long long)xlate64(dlocn->offset));
+				  (unsigned long long)htole64(dlocn->offset));
 			log_print("pv_header.disk_locn[%d].size %llu", di,
-				  (unsigned long long)xlate64(dlocn->size));
+				  (unsigned long long)htole64(dlocn->size));
 		}
 		di++;
 		dlocn++;
@@ -1110,9 +1102,9 @@ static int _dump_label_and_pv_header(struct cmd_context *cmd, uint64_t labelsect
 		log_print("pv_header.disk_locn[%d] at %llu # location list end", di,
 			  (unsigned long long) dlocn_offset);
 		log_print("pv_header.disk_locn[%d].offset %llu", di,
-			  (unsigned long long)xlate64(dlocn->offset));
+			  (unsigned long long)htole64(dlocn->offset));
 		log_print("pv_header.disk_locn[%d].size %llu", di,
-			  (unsigned long long)xlate64(dlocn->size));
+			  (unsigned long long)htole64(dlocn->size));
 	}
 
 	/* advance past the all-zero dlocn struct */
@@ -1124,31 +1116,34 @@ static int _dump_label_and_pv_header(struct cmd_context *cmd, uint64_t labelsect
 	if ((void *)dlocn != (void *)(buf + dlocn_offset - lh_offset))
 		log_print("CHECK: problem with pv_header.disk_locn[%d] offset calculation", di);
 
-	while ((tmp = xlate64(dlocn->offset))) {
+	while (htole64(dlocn->offset)) {
 		if (print_fields) {
 			log_print("pv_header.disk_locn[%d] at %llu # location of metadata area", di,
 				  (unsigned long long)dlocn_offset);
 			log_print("pv_header.disk_locn[%d].offset %llu", di,
-				  (unsigned long long)xlate64(dlocn->offset));
+				  (unsigned long long)htole64(dlocn->offset));
 			log_print("pv_header.disk_locn[%d].size %llu", di,
-				  (unsigned long long)xlate64(dlocn->size));
+				  (unsigned long long)htole64(dlocn->size));
 		}
 
 		if (!mda_count) {
-			*mda1_offset = xlate64(dlocn->offset);
-			*mda1_size = xlate64(dlocn->size);
+			*mda1_offset = htole64(dlocn->offset);
+			*mda1_size = htole64(dlocn->size);
 
 			/*
 			 * mda1 offset is page size from machine that created it,
 			 * warn if it's not one of the expected page sizes.
 			 */
-			if ((*mda1_offset != 4096) && (*mda1_offset != 8192) && (*mda1_offset != 65536)) {
+			if ((*mda1_offset != 4096) && 
+			    (*mda1_offset != 8192) &&
+			    (*mda1_offset != 16384) &&
+			    (*mda1_offset != 65536)) {
 				log_print("WARNING: pv_header.disk_locn[%d].offset %llu is unexpected # for first mda",
 					  di, (unsigned long long)*mda1_offset);
 			}
 		} else {
-			*mda2_offset = xlate64(dlocn->offset);
-			*mda2_size = xlate64(dlocn->size);
+			*mda2_offset = htole64(dlocn->offset);
+			*mda2_size = htole64(dlocn->size);
 
 			/*
 			 * No fixed location for second mda, so we have to look for
@@ -1169,9 +1164,9 @@ static int _dump_label_and_pv_header(struct cmd_context *cmd, uint64_t labelsect
 		log_print("pv_header.disk_locn[%d] at %llu # location list end", di,
 			  (unsigned long long) dlocn_offset);
 		log_print("pv_header.disk_locn[%d].offset %llu", di,
-			  (unsigned long long)xlate64(dlocn->offset));
+			  (unsigned long long)htole64(dlocn->offset));
 		log_print("pv_header.disk_locn[%d].size %llu", di,
-			  (unsigned long long)xlate64(dlocn->size));
+			  (unsigned long long)htole64(dlocn->size));
 	}
 
 	/* advance past the all-zero dlocn struct */
@@ -1194,8 +1189,8 @@ static int _dump_label_and_pv_header(struct cmd_context *cmd, uint64_t labelsect
 
 	if (print_fields) {
 		log_print("pv_header_extension at %llu", (unsigned long long)pvhe_offset);
-		log_print("pv_header_extension.version %u", xlate32(pvhe->version));
-		log_print("pv_header_extension.flags %u", xlate32(pvhe->flags));
+		log_print("pv_header_extension.version %u", htole32(pvhe->version));
+		log_print("pv_header_extension.flags %u", htole32(pvhe->flags));
 	}
 
 	/*
@@ -1207,14 +1202,14 @@ static int _dump_label_and_pv_header(struct cmd_context *cmd, uint64_t labelsect
 	dlocn = pvhe->bootloader_areas_xl;
 	dlocn_offset = pvhe_offset + 8;
 
-	while ((tmp = xlate64(dlocn->offset))) {
+	while (htole64(dlocn->offset)) {
 		if (print_fields) {
 			log_print("pv_header_extension.disk_locn[%d] at %llu # bootloader area", di,
 				  (unsigned long long)dlocn_offset);
 			log_print("pv_header_extension.disk_locn[%d].offset %llu", di,
-				  (unsigned long long)xlate64(dlocn->offset));
+				  (unsigned long long)htole64(dlocn->offset));
 			log_print("pv_header_extension.disk_locn[%d].size %llu", di,
-				  (unsigned long long)xlate64(dlocn->size));
+				  (unsigned long long)htole64(dlocn->size));
 		}
 
 		di++;
@@ -1227,9 +1222,9 @@ static int _dump_label_and_pv_header(struct cmd_context *cmd, uint64_t labelsect
 		log_print("pv_header_extension.disk_locn[%d] at %llu # location list end", di,
 			  (unsigned long long) dlocn_offset);
 		log_print("pv_header_extension.disk_locn[%d].offset %llu", di,
-			  (unsigned long long)xlate64(dlocn->offset));
+			  (unsigned long long)htole64(dlocn->offset));
 		log_print("pv_header_extension.disk_locn[%d].size %llu", di,
-			  (unsigned long long)xlate64(dlocn->size));
+			  (unsigned long long)htole64(dlocn->size));
 	}
 
 	if (bad)
@@ -1260,7 +1255,7 @@ static int _dump_mda_header(struct cmd_context *cmd, struct settings *set,
 			    uint32_t *checksum0_ret,
 			    int *found_header)
 {
-	char buf[512];
+	char buf[512 + 1] = { 0 };
 	char str[256];
 	char *mda_buf;
 	struct mda_header *mh;
@@ -1269,7 +1264,7 @@ static int _dump_mda_header(struct cmd_context *cmd, struct settings *set,
 	uint64_t meta_offset = 0; /* bytes */
 	uint64_t meta_size = 0;   /* bytes */
 	uint32_t meta_checksum = 0;
-	int mda_num = (mda_offset == 4096) ? 1 : 2;
+	int mda_num = (mda_offset <= 65536) ? 1 : 2;
 	int bad = 0;
 
 	*checksum0_ret = 0; /* checksum from raw_locn[0] */
@@ -1294,11 +1289,11 @@ static int _dump_mda_header(struct cmd_context *cmd, struct settings *set,
 
 	if (print_fields) {
 		log_print("mda_header_%d at %llu # metadata area", mda_num, (unsigned long long)mda_offset);
-		log_print("mda_header_%d.checksum 0x%x", mda_num, xlate32(mh->checksum_xl));
+		log_print("mda_header_%d.checksum 0x%x", mda_num, htole32(mh->checksum_xl));
 		log_print("mda_header_%d.magic 0x%s", mda_num, _chars_to_hexstr(mh->magic, str, 16, 256, "mda_header.magic"));
-		log_print("mda_header_%d.version %u", mda_num, xlate32(mh->version));
-		log_print("mda_header_%d.start %llu", mda_num, (unsigned long long)xlate64(mh->start));
-		log_print("mda_header_%d.size %llu", mda_num, (unsigned long long)xlate64(mh->size));
+		log_print("mda_header_%d.version %u", mda_num, htole32(mh->version));
+		log_print("mda_header_%d.start %llu", mda_num, (unsigned long long)htole64(mh->start));
+		log_print("mda_header_%d.size %llu", mda_num, (unsigned long long)htole64(mh->size));
 	}
 
 	if (!_check_mda_header(mh, mda_num, mda_offset, mda_size, found_header))
@@ -1359,7 +1354,7 @@ static int _dump_mda_header(struct cmd_context *cmd, struct settings *set,
 	 * looking at all copies of the metadata in the area
 	 */
 	if (print_metadata == PRINT_ALL) {
-		if (!(mda_buf = zalloc(mda_size)))
+		if (!(mda_buf = zalloc(mda_size + 1)))
 			goto_out;
 
 		if (!_read_bytes(dev, def, mda_offset, mda_size, mda_buf)) {
@@ -1437,8 +1432,13 @@ static int _dump_metadata(struct cmd_context *cmd, const char *dump, struct sett
 	int bad = 0;
 
 	if (arg_is_set(cmd, file_ARG)) {
+		struct stat sb;
 		if (!(tofile = arg_str_value(cmd, file_ARG, NULL)))
 			return 0;
+		if (!stat(tofile, &sb)) {
+			log_error("File already exists.");
+			return 0;
+		}
 	}
 
 	if (set->mda_num)
@@ -1736,7 +1736,7 @@ static int _dump_search(struct cmd_context *cmd, const char *dump, struct settin
 	log_print("Searching for metadata at offset %llu size %llu",
 		  (unsigned long long)mda_offset, (unsigned long long)mda_size);
 
-	if (!(buf = zalloc(mda_size)))
+	if (!(buf = zalloc(mda_size + 1)))
 		return_0;
 
 	if (!_read_bytes(dev, def, mda_offset, mda_size, buf)) {
@@ -1756,74 +1756,74 @@ static int _dump_search(struct cmd_context *cmd, const char *dump, struct settin
 
 static int _get_one_setting(struct cmd_context *cmd, struct settings *set, char *key, char *val)
 {
-	if (!strncmp(key, "metadata_offset", strlen("metadata_offset"))) {
+	if (!strncmp(key, "metadata_offset", sizeof("metadata_offset") - 1)) {
 		if (sscanf(val, "%llu", (unsigned long long *)&set->metadata_offset) != 1)
 			goto_bad;
 		set->metadata_offset_set = 1;
 		return 1;
 	}
 
-	if (!strncmp(key, "seqno", strlen("seqno"))) {
+	if (!strncmp(key, "seqno", sizeof("seqno") - 1)) {
 		if (sscanf(val, "%u", &set->seqno) != 1)
 			goto_bad;
 		set->seqno_set = 1;
 		return 1;
 	}
 
-	if (!strncmp(key, "backup_file", strlen("backup_file"))) {
+	if (!strncmp(key, "backup_file", sizeof("backup_file") - 1)) {
 		if ((set->backup_file = dm_pool_strdup(cmd->mem, val)))
 			return 1;
 		return 0;
 	}
 
-	if (!strncmp(key, "mda_offset", strlen("mda_offset"))) {
+	if (!strncmp(key, "mda_offset", sizeof("mda_offset") - 1)) {
 		if (sscanf(val, "%llu", (unsigned long long *)&set->mda_offset) != 1)
 			goto_bad;
 		set->mda_offset_set = 1;
 		return 1;
 	}
 
-	if (!strncmp(key, "mda_size", strlen("mda_size"))) {
+	if (!strncmp(key, "mda_size", sizeof("mda_size") - 1)) {
 		if (sscanf(val, "%llu", (unsigned long long *)&set->mda_size) != 1)
 			goto_bad;
 		set->mda_size_set = 1;
 		return 1;
 	}
 
-	if (!strncmp(key, "mda2_offset", strlen("mda2_offset"))) {
+	if (!strncmp(key, "mda2_offset", sizeof("mda2_offset") - 1)) {
 		if (sscanf(val, "%llu", (unsigned long long *)&set->mda2_offset) != 1)
 			goto_bad;
 		set->mda2_offset_set = 1;
 		return 1;
 	}
 
-	if (!strncmp(key, "mda2_size", strlen("mda2_size"))) {
+	if (!strncmp(key, "mda2_size", sizeof("mda2_size") - 1)) {
 		if (sscanf(val, "%llu", (unsigned long long *)&set->mda2_size) != 1)
 			goto_bad;
 		set->mda2_size_set = 1;
 		return 1;
 	}
 
-	if (!strncmp(key, "device_size", strlen("device_size"))) {
+	if (!strncmp(key, "device_size", sizeof("device_size") - 1)) {
 		if (sscanf(val, "%llu", (unsigned long long *)&set->device_size) != 1)
 			goto_bad;
 		set->device_size_set = 1;
 		return 1;
 	}
 
-	if (!strncmp(key, "data_offset", strlen("data_offset"))) {
+	if (!strncmp(key, "data_offset", sizeof("data_offset") - 1)) {
 		if (sscanf(val, "%llu", (unsigned long long *)&set->data_offset) != 1)
 			goto_bad;
 		set->data_offset_set = 1;
 		return 1;
 	}
 
-	if (!strncmp(key, "pv_uuid", strlen("pv_uuid"))) {
+	if (!strncmp(key, "pv_uuid", sizeof("pv_uuid") - 1)) {
 		if (strchr(val, '-') && (strlen(val) == 32)) {
-			memcpy(&set->pvid, val, 32);
+			memcpy(&set->pv_id, val, 32);
 			set->pvid_set = 1;
 			return 1;
-		} else if (id_read_format_try(&set->pvid, val)) {
+		} else if (id_read_format_try(&set->pv_id, val)) {
 			set->pvid_set = 1;
 			return 1;
 		} else {
@@ -1832,7 +1832,7 @@ static int _get_one_setting(struct cmd_context *cmd, struct settings *set, char
 		}
 	}
 
-	if (!strncmp(key, "mda_num", strlen("mda_num"))) {
+	if (!strncmp(key, "mda_num", sizeof("mda_num") - 1)) {
 		if (sscanf(val, "%u", (int *)&set->mda_num) != 1)
 			goto_bad;
 		return 1;
@@ -1848,8 +1848,8 @@ static int _get_settings(struct cmd_context *cmd, struct settings *set)
 	const char *str;
 	char key[64];
 	char val[64];
-	int num;
-	int pos;
+	unsigned num;
+	unsigned pos;
 
 	/*
 	 * "grouped" means that multiple --settings options can be used.
@@ -1915,7 +1915,7 @@ static int _repair_label_header(struct cmd_context *cmd, const char *repair,
 
 	if (!found_label) {
 		log_warn("WARNING: No LVM label found on %s.  It may not be an LVM device.", dev_name(dev));
-		if (!arg_count(cmd, yes_ARG) &&
+		if (!arg_is_set(cmd, yes_ARG) &&
 		    yes_no_prompt("Write LVM header to device? ") == 'n')
 			return 0;
 	}
@@ -1937,13 +1937,13 @@ static int _repair_label_header(struct cmd_context *cmd, const char *repair,
 
 	memcpy(lh->id, LABEL_ID, sizeof(lh->id));
 	memcpy(lh->type, LVM2_LABEL, sizeof(lh->type));
-	lh->sector_xl =  xlate64(labelsector);
-	lh->offset_xl = xlate32(32);
+	lh->sector_xl =  htole64(labelsector);
+	lh->offset_xl = htole32(32);
 
 	crc = calc_crc(INITIAL_CRC, (uint8_t *)&lh->offset_xl,
 	               LABEL_SIZE - ((uint8_t *) &lh->offset_xl - (uint8_t *) lh));
 
-	lh->crc_xl = xlate32(crc);
+	lh->crc_xl = htole32(crc);
        
 	log_print("Writing label_header.crc 0x%08x", crc);
 
@@ -1952,7 +1952,7 @@ static int _repair_label_header(struct cmd_context *cmd, const char *repair,
 		return 1;
 	}
 
-	if (!arg_count(cmd, yes_ARG) &&
+	if (!arg_is_set(cmd, yes_ARG) &&
 	    yes_no_prompt("Write new LVM header to %s? ", dev_name(dev)) == 'n')
 		return 0;
 
@@ -1971,19 +1971,15 @@ static int _get_pv_info_from_metadata(struct cmd_context *cmd, struct settings *
 				     uint64_t *device_size_sectors,
 				     uint64_t *pe_start_sectors)
 {
-	int8_t pvid_cur[ID_LEN+1];  /* found in existing pv_header */
-	int8_t pvid_set[ID_LEN+1];  /* set by user in --settings */
-	int8_t pvid_use[ID_LEN+1];  /* the pvid chosen to use */
-	int pvid_cur_valid = 0;     /* pvid_cur is valid */
-	int pvid_use_valid = 0;     /* pvid_use is valid */
+	char pvid_cur[ID_LEN + 1] = { 0 };  /* found in existing pv_header */
+	char pvid_set[ID_LEN + 1] = { 0 };  /* set by user in --settings */
+	char pvid_use[ID_LEN + 1] = { 0 };  /* the pvid chosen to use */
+	int pvid_cur_valid = 0;             /* pvid_cur is valid */
+	int pvid_use_valid = 0;             /* pvid_use is valid */
 	struct dm_config_tree *cft = NULL;
 	struct volume_group *vg = NULL;
 	struct pv_list *pvl;
 
-	memset(pvid_cur, 0, sizeof(pvid_cur));
-	memset(pvid_set, 0, sizeof(pvid_set));
-	memset(pvid_use, 0, sizeof(pvid_use));
-
 	/*
 	 * Check if there's a valid existing PV UUID at the expected location.
 	 */
@@ -1996,14 +1992,14 @@ static int _get_pv_info_from_metadata(struct cmd_context *cmd, struct settings *
 	}
 
 	if (set->pvid_set) {
-		memcpy(&pvid_set, &set->pvid, ID_LEN);
+		memcpy(&pvid_set, &set->pv_id, ID_LEN);
 		memcpy(&pvid_use, &pvid_set, ID_LEN);
 		pvid_use_valid = 1;
 	}
 
 	if (pvid_cur_valid && set->pvid_set && memcmp(&pvid_cur, &pvid_set, ID_LEN)) {
-		log_warn("WARNING: existing PV UUID %s does not match pv_uuid setting %s.",
-			 (char *)&pvid_cur, (char *)&pvid_set);
+		log_warn("WARNING: Existing PV UUID %s does not match pv_uuid setting %s.",
+			 pvid_cur, pvid_set);
 
 		memcpy(&pvid_use, &pvid_set, ID_LEN);
 		pvid_use_valid = 1;
@@ -2041,7 +2037,7 @@ static int _get_pv_info_from_metadata(struct cmd_context *cmd, struct settings *
 		}
 	} else {
 		dm_list_iterate_items(pvl, &vg->pvs) {
-			if (id_equal(&pvl->pv->id, (struct id *)&pvid_use))
+			if (!memcmp(&pvl->pv->id.uuid, &pvid_use, ID_LEN))
 				goto copy_pv;
 		}
 	}
@@ -2055,9 +2051,9 @@ static int _get_pv_info_from_metadata(struct cmd_context *cmd, struct settings *
 	 * . the metadata has no PV with a device name hint matching this device
 	 */
 	if (set->pvid_set)
-		log_error("PV UUID %s not found in metadata file.", (char *)&pvid_set);
+		log_error("PV UUID %s not found in metadata file.", pvid_set);
 	else if (pvid_cur_valid)
-		log_error("PV UUID %s in existing pv_header not found in metadata file.", (char *)&pvid_cur);
+		log_error("PV UUID %s in existing pv_header not found in metadata file.", pvid_cur);
 	else if (!pvid_use_valid)
 		log_error("PV name %s not found in metadata file.", dev_name(dev));
 
@@ -2112,7 +2108,7 @@ static int _check_for_mda2(struct cmd_context *cmd, struct device *dev,
 	char buf2[256];
 	char *buf;
 	uint64_t mda_offset, mda_size, extra_bytes; /* bytes */
-	int i, found = 0;
+	unsigned i, found = 0;
 
 	if (device_size < (2 * ONE_MB_IN_BYTES))
 		return_0;
@@ -2137,12 +2133,12 @@ static int _check_for_mda2(struct cmd_context *cmd, struct device *dev,
 	if (memcmp(mh->magic, FMTT_MAGIC, sizeof(mh->magic)))
 		goto fail;
 
-	if (xlate32(mh->version) != FMTT_VERSION) {
+	if (htole32(mh->version) != FMTT_VERSION) {
 		log_print("Skipping mda2 (wrong mda_header.version)");
 		goto fail;
 	}
 
-	if (xlate64(mh->start) != mda_offset) {
+	if (htole64(mh->start) != mda_offset) {
 		log_print("Skipping mda2 (wrong mda_header.start)");
 		goto fail;
 	}
@@ -2216,7 +2212,7 @@ static int _repair_pv_header(struct cmd_context *cmd, const char *repair,
 			     uint64_t labelsector, struct device *dev)
 {
 	char head_buf[512];
-	int8_t pvid[ID_LEN+1];
+	char pvid[ID_LEN + 1] __attribute__((aligned(8))) = { 0 };
 	struct device *dev_with_pvid = NULL;
 	struct label_header *lh;
 	struct pv_header *pvh;
@@ -2234,8 +2230,6 @@ static int _repair_pv_header(struct cmd_context *cmd, const char *repair,
 	int found_label = 0;
 	int di;
 
-	memset(&pvid, 0, ID_LEN+1);
-
 	lh_offset = labelsector * 512; /* from start of disk */
 
 	if (!dev_get_size(dev, &get_size_sectors))
@@ -2293,9 +2287,9 @@ static int _repair_pv_header(struct cmd_context *cmd, const char *repair,
 		log_warn("WARNING: PV will have no metadata with zero metadata areas.");
 
 	} else if (!mda_count) {
-		log_warn("WARNING: no previous metadata areas found on device.");
+		log_warn("WARNING: No previous metadata areas found on device.");
 
-		if (arg_count(cmd, yes_ARG) ||
+		if (arg_is_set(cmd, yes_ARG) ||
 		    yes_no_prompt("Should a metadata area be included? ") == 'y') {
 			/* mda1_offset/mda1_size are set below */
 			mda_count = 1;
@@ -2346,7 +2340,7 @@ static int _repair_pv_header(struct cmd_context *cmd, const char *repair,
 	if (set->device_size_set && set->pvid_set && set->data_offset_set && !mf->filename) {
 		device_size = set->device_size;
 		pe_start_sectors = set->data_offset >> SECTOR_SHIFT;
-		memcpy(&pvid, &set->pvid, ID_LEN);
+		memcpy(pvid, &set->pv_id, ID_LEN);
 
 		if (get_size && (get_size != device_size)) {
 			log_warn("WARNING: device_size setting %llu bytes does not match device size %llu bytes.",
@@ -2378,7 +2372,7 @@ static int _repair_pv_header(struct cmd_context *cmd, const char *repair,
 	 * dev_size and pe_start from the metadata to use in the pv_header.
 	 */
 	if (!_get_pv_info_from_metadata(cmd, set, dev, pvh, found_label,
-					mf->text_buf, mf->text_size, (char *)&pvid,
+					mf->text_buf, mf->text_size, pvid,
 					&device_size_sectors, &pe_start_sectors))
 		goto fail;
 
@@ -2392,13 +2386,13 @@ static int _repair_pv_header(struct cmd_context *cmd, const char *repair,
 	 * Read all devs to verify the pvid that will be written does not exist
 	 * on another device.
 	 */
-	if (!label_scan_for_pvid(cmd, (char *)&pvid, &dev_with_pvid)) {
+	if (!label_scan_for_pvid(cmd, pvid, &dev_with_pvid)) {
 		log_error("Failed to scan devices to check PV UUID.");
 		goto fail;
 	}
 
 	if (dev_with_pvid && (dev_with_pvid != dev)) {
-		log_error("Cannot use PV UUID %s which exists on %s", (char *)&pvid, dev_name(dev_with_pvid));
+		log_error("Cannot use PV UUID %s which exists on %s", pvid, dev_name(dev_with_pvid));
 		goto fail;
 	}
 
@@ -2409,16 +2403,16 @@ static int _repair_pv_header(struct cmd_context *cmd, const char *repair,
 	/* set label_header (except crc) */
 	memcpy(lh->id, LABEL_ID, sizeof(lh->id));
 	memcpy(lh->type, LVM2_LABEL, sizeof(lh->type));
-	lh->sector_xl =  xlate64(labelsector);
-	lh->offset_xl = xlate32(32);
+	lh->sector_xl =  htole64(labelsector);
+	lh->offset_xl = htole32(32);
 
 	/* set pv_header */
 	memcpy(pvh->pv_uuid, &pvid, ID_LEN);
-	pvh->device_size_xl = xlate64(device_size);
+	pvh->device_size_xl = htole64(device_size);
 
 	/* set data area location */
 	data_offset = (pe_start_sectors << SECTOR_SHIFT);
-	pvh->disk_areas_xl[0].offset = xlate64(data_offset);
+	pvh->disk_areas_xl[0].offset = htole64(data_offset);
 	pvh->disk_areas_xl[0].size = 0;
 
 	/* set end of data areas */
@@ -2431,15 +2425,15 @@ static int _repair_pv_header(struct cmd_context *cmd, const char *repair,
 	if (mda_count > 0) {
 		mda1_offset = 4096;
 		mda1_size = (pe_start_sectors << SECTOR_SHIFT) - 4096;
-		pvh->disk_areas_xl[di].offset = xlate64(mda1_offset);
-		pvh->disk_areas_xl[di].size = xlate64(mda1_size);
+		pvh->disk_areas_xl[di].offset = htole64(mda1_offset);
+		pvh->disk_areas_xl[di].size = htole64(mda1_size);
 		di++;
 	}
 
 	/* set second metadata area location */
 	if (mda_count > 1) {
-		pvh->disk_areas_xl[di].offset = xlate64(mda2_offset);
-		pvh->disk_areas_xl[di].size = xlate64(mda2_size);
+		pvh->disk_areas_xl[di].offset = htole64(mda2_offset);
+		pvh->disk_areas_xl[di].size = htole64(mda2_size);
 		di++;
 	}
 
@@ -2450,8 +2444,8 @@ static int _repair_pv_header(struct cmd_context *cmd, const char *repair,
 
 	/* set pv_header_extension */
 	pvhe = (struct pv_header_extension *)((char *)pvh + sizeof(struct pv_header) + (di * sizeof(struct disk_locn)));
-	pvhe->version = xlate32(PV_HEADER_EXTENSION_VSN);
-	pvhe->flags = xlate32(PV_EXT_USED);
+	pvhe->version = htole32(PV_HEADER_EXTENSION_VSN);
+	pvhe->flags = htole32(PV_EXT_USED);
 	pvhe->bootloader_areas_xl[0].offset = 0;
 	pvhe->bootloader_areas_xl[0].size = 0;
 
@@ -2459,14 +2453,14 @@ static int _repair_pv_header(struct cmd_context *cmd, const char *repair,
 			    LABEL_SIZE - ((uint8_t *) &lh->offset_xl - (uint8_t *) lh));
 
 	/* set label_header crc (last) */
-	lh->crc_xl = xlate32(head_crc);
+	lh->crc_xl = htole32(head_crc);
 
 	/*
 	 * Write the updated header sector.
 	 */
 
 	log_print("Writing label_header.crc 0x%08x pv_header uuid %s device_size %llu",
-		  head_crc, (char *)&pvid, (unsigned long long)device_size);
+		  head_crc, pvid, (unsigned long long)device_size);
 
 	log_print("Writing data_offset %llu mda1_offset %llu mda1_size %llu mda2_offset %llu mda2_size %llu",
 		  (unsigned long long)data_offset,
@@ -2480,7 +2474,7 @@ static int _repair_pv_header(struct cmd_context *cmd, const char *repair,
 		return 1;
 	}
 
-	if (!arg_count(cmd, yes_ARG) &&
+	if (!arg_is_set(cmd, yes_ARG) &&
 	    yes_no_prompt("Write new LVM header to %s? ", dev_name(dev)) == 'n')
 		goto fail;
 
@@ -2525,15 +2519,15 @@ static int _update_mda(struct cmd_context *cmd, struct metadata_file *mf, struct
 
 	mh = (struct mda_header *)buf;
 	memcpy(mh->magic, FMTT_MAGIC, sizeof(mh->magic));
-	mh->version = xlate32(FMTT_VERSION);
-	mh->start = xlate64(mda_offset);
-	mh->size = xlate64(mda_size);
+	mh->version = htole32(FMTT_VERSION);
+	mh->start = htole64(mda_offset);
+	mh->size = htole64(mda_size);
 
 	rlocn0 = mh->raw_locns;
 	rlocn0->flags = 0;
-	rlocn0->offset = xlate64(512); /* text begins 512 from start of mda_header */
-	rlocn0->size = xlate64(mf->text_size);
-	rlocn0->checksum = xlate32(mf->text_crc);
+	rlocn0->offset = htole64(512); /* text begins 512 from start of mda_header */
+	rlocn0->size = htole64(mf->text_size);
+	rlocn0->checksum = htole32(mf->text_crc);
 
 	rlocn1 = (struct raw_locn *)((char *)mh->raw_locns + 24);
 	rlocn1->flags = 0;
@@ -2543,7 +2537,7 @@ static int _update_mda(struct cmd_context *cmd, struct metadata_file *mf, struct
 
 	crc = calc_crc(INITIAL_CRC, (uint8_t *)mh->magic,
 		       MDA_HEADER_SIZE - sizeof(mh->checksum_xl));
-	mh->checksum_xl = xlate32(crc);
+	mh->checksum_xl = htole32(crc);
 
 	log_print("Writing metadata at %llu length %llu crc 0x%08x mda%d",
 		  (unsigned long long)(mda_offset + 512),
@@ -2557,7 +2551,7 @@ static int _update_mda(struct cmd_context *cmd, struct metadata_file *mf, struct
 		return 1;
 	}
 
-	if (!arg_count(cmd, yes_ARG) &&
+	if (!arg_is_set(cmd, yes_ARG) &&
 	    yes_no_prompt("Write new LVM metadata to %s? ", dev_name(dev)) == 'n')
 		goto fail;
 
@@ -2828,7 +2822,7 @@ static int _dump_backup_to_raw(struct cmd_context *cmd, struct settings *set)
 		goto fail_close;
 
 	rv = read(fd, back_buf, back_size);
-	if (rv != back_size) {
+	if (rv != (int)back_size) {
 		log_error("Cannot read file: %s", input);
 		free(back_buf);
 		goto fail_close;
@@ -2914,7 +2908,7 @@ static int _check_metadata_file(struct cmd_context *cmd, struct metadata_file *m
 	if (text_buf[text_size-1] != '\0' ||
 	    text_buf[text_size-2] != '\n' ||
 	    text_buf[text_size-3] != '\n')
-		log_warn("WARNING: unexpected final bytes of raw metadata, expected \\n\\n\\0.");
+		log_warn("WARNING: Unexpected final bytes of raw metadata, expected \\n\\n\\0.");
 
 	if (_check_vgname_start(text_buf, &namelen)) {
 		if (!(vgid = strstr(text_buf, "id = "))) {
@@ -2925,9 +2919,9 @@ static int _check_metadata_file(struct cmd_context *cmd, struct metadata_file *m
 		return 1;
 	}
 
-	log_warn("WARNING: file data does not begin with a VG name and may be invalid.");
+	log_warn("WARNING: File data does not begin with a VG name and may be invalid.");
 
-	if (!arg_count(cmd, yes_ARG) &&
+	if (!arg_is_set(cmd, yes_ARG) &&
 	    yes_no_prompt("Write input file data to disk?") == 'n') {
 		log_error("Invalid raw text metadata in file.");
 		return 0;
@@ -2961,17 +2955,16 @@ static int _read_metadata_file(struct cmd_context *cmd, struct metadata_file *mf
 		goto out;
 	}
 
-	if (!(text_buf = zalloc(text_size + 1)))
+	if (!(text_buf = malloc(text_size + 1)))
 		goto_out;
 
 	rv = read(fd, text_buf, text_size);
-	if (rv != text_size) {
+	if (rv != (int)text_size) {
 		log_error("Cannot read file: %s", mf->filename);
 		free(text_buf);
 		goto out;
 	}
-
-	text_size += 1; /* null terminating byte */
+	text_buf[text_size++] = 0; /* null terminating byte */
 
 	if (close(fd))
 		stack;
@@ -3006,10 +2999,9 @@ out:
 	return 0;
 }
 
-int pvck(struct cmd_context *cmd, int argc, char **argv)
+static int _pvck_mf(struct metadata_file *mf, struct cmd_context *cmd, int argc, char **argv)
 {
-	struct settings set;
-	struct metadata_file mf;
+	struct settings set = { 0 };
 	struct device *dev = NULL;
 	struct devicefile *def = NULL;
 	const char *dump, *repair;
@@ -3019,9 +3011,6 @@ int pvck(struct cmd_context *cmd, int argc, char **argv)
 	int ret = 0;
 	int i;
 
-	memset(&set, 0, sizeof(set));
-	memset(&mf, 0, sizeof(mf));
-
 	/*
 	 * By default LVM skips the first sector (sector 0), and writes
 	 * the label_header in the second sector (sector 1).
@@ -3077,10 +3066,10 @@ int pvck(struct cmd_context *cmd, int argc, char **argv)
 		return_ECMD_FAILED;
 
 	if (arg_is_set(cmd, file_ARG) && (arg_is_set(cmd, repairtype_ARG) || arg_is_set(cmd, repair_ARG))) {
-		if (!(mf.filename = arg_str_value(cmd, file_ARG, NULL)))
+		if (!(mf->filename = arg_str_value(cmd, file_ARG, NULL)))
 			return_ECMD_FAILED;
 
-		if (!_read_metadata_file(cmd, &mf))
+		if (!_read_metadata_file(cmd, mf))
 			return_ECMD_FAILED;
 	}
 
@@ -3153,10 +3142,10 @@ int pvck(struct cmd_context *cmd, int argc, char **argv)
 			ret = _repair_label_header(cmd, repair, &set, labelsector, dev);
 
 		else if (!strcmp(repair, "pv_header"))
-			ret = _repair_pv_header(cmd, repair, &set, &mf, labelsector, dev);
+			ret = _repair_pv_header(cmd, repair, &set, mf, labelsector, dev);
 
 		else if (!strcmp(repair, "metadata"))
-			ret = _repair_metadata(cmd, repair, &set, &mf, labelsector, dev);
+			ret = _repair_metadata(cmd, repair, &set, mf, labelsector, dev);
 		else
 			log_error("Unknown repair value.");
 
@@ -3170,10 +3159,10 @@ int pvck(struct cmd_context *cmd, int argc, char **argv)
 
 		/* repair is a combination of repairtype pv_header+metadata */
 
-		if (!_repair_pv_header(cmd, "pv_header", &set, &mf, labelsector, dev))
+		if (!_repair_pv_header(cmd, "pv_header", &set, mf, labelsector, dev))
 			return_ECMD_FAILED;
 
-		if (!_repair_metadata(cmd, "metadata", &set, &mf, labelsector, dev))
+		if (!_repair_metadata(cmd, "metadata", &set, mf, labelsector, dev))
 			return_ECMD_FAILED;
 
 		return ECMD_PROCESSED;
@@ -3184,9 +3173,10 @@ int pvck(struct cmd_context *cmd, int argc, char **argv)
 	 * but this is here to preserve the historical output.
 	 */
 
-	if (argc == 1)
-		setup_device(cmd, argv[0]);
-	else if (!setup_devices(cmd))
+	if (argc == 1) {
+		if (!setup_device(cmd, argv[0]))
+			return_ECMD_FAILED;
+	} else if (!setup_devices(cmd))
 		return_ECMD_FAILED;
 
 	for (i = 0; i < argc; i++) {
@@ -3205,3 +3195,12 @@ int pvck(struct cmd_context *cmd, int argc, char **argv)
 		return_ECMD_FAILED;
 	return ECMD_PROCESSED;
 }
+
+int pvck(struct cmd_context *cmd, int argc, char **argv)
+{
+	struct metadata_file mf = { 0 };
+	int ret = _pvck_mf(&mf, cmd, argc, argv);
+
+	free(mf.text_buf);
+	return ret;
+}
